"""Unit tests for the FunctionAssigner class"""

# Copyright (C) 2013 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2013-11-07
# Last changed: 2013-11-07

import pytest
from dolfin import *
import numpy as np

from dolfin_utils.test import fixture as fixt

@fixt
def mesh():
    return UnitCubeMesh(8, 8, 8)

@fixt
def V(mesh):
    return FunctionSpace(mesh, "CG", 1)

@fixt
def Q(mesh):
    return FunctionSpace(mesh, "CG", 2)

@fixt
def W(mesh):
    return VectorFunctionSpace(mesh, "CG", 1)

@fixt
def QQ(mesh):
    return VectorFunctionSpace(mesh, "CG", 2)

@fixt
def R(mesh):
    return FunctionSpace(mesh, "R", 0)

@fixt
def RR(mesh):
    return VectorFunctionSpace(mesh, "R", 0)

@fixt
def QQV(mesh):
    QQ = VectorElement("CG", mesh.ufl_cell(), 2)
    V = FiniteElement("CG", mesh.ufl_cell(), 1)
    return FunctionSpace(mesh, QQ*V)

@fixt
def WW(mesh):
    W = VectorElement("CG", mesh.ufl_cell(), 1)
    return FunctionSpace(mesh, W*W)

@fixt
def WR(mesh):
    W = VectorElement("CG", mesh.ufl_cell(), 1)
    R = FiniteElement("R", mesh.ufl_cell(), 0)
    return FunctionSpace(mesh, W*R)

@fixt
def WRR(mesh):
    W = VectorElement("CG", mesh.ufl_cell(), 1)
    RR = VectorElement("R", mesh.ufl_cell(), 0)
    return FunctionSpace(mesh, W*RR)

@fixt
def u0(V):
     u0_ = Function(V)
     u0_.vector()[:] = 1.
     return u0_

@fixt
def u1(V):
     u1_ = Function(V)
     u1_.vector()[:] = 2.
     return u1_

@fixt
def u2(V):
     u2_ = Function(V)
     u2_.vector()[:] = 3.
     return u2_

@fixt
def r(R):
    r_ = Function(R)
    r_.vector()[:] = 3.
    return r_

@fixt
def rr(RR):
    rr_ = Function(RR)
    rr_.vector()[:] = 2.
    return rr_

@fixt
def w(W):
    w_ = Function(W)
    w_.vector()[:] = 4.
    return w_

@fixt
def ww(WW):
    ww_ = Function(WW)
    ww_.vector()[:] = 5.
    return ww_

@fixt
def wr(WR):
    wr_ = Function(WR)
    wr_.vector()[:] = 6.
    return wr_

@fixt
def wrr(WRR):
    wrr_ = Function(WRR)
    wrr_.vector()[:] = 7.
    return wrr_

@fixt
def q(Q):
    q_ = Function(Q)
    q_.vector()[:] = 1.
    return q_

@fixt
def qq(QQ):
    qq_ = Function(QQ)
    qq_.vector()[:] = 2.
    return qq_

@fixt
def qqv(QQV):
    qqv_ = Function(QQV)
    qqv_.vector()[:] = 3.
    return qqv_


def test_1_1_assigner(w, ww, wr, wrr, q, r, qqv, u0, u1, u2, W, V, WW):

    assigner = FunctionAssigner(W.sub(0), V)
    assigner.assign(w.sub(0), u0)

    assert np.all(w.sub(0, deepcopy=True).vector().get_local() == u0.vector().get_local())

    assign(w.sub(2), u2)
    assert np.all(w.sub(2, deepcopy=True).vector().get_local() == u2.vector().get_local())

    assigner = FunctionAssigner(V, W.sub(2))
    assigner.assign(u0, w.sub(2))

    assert np.all(u0.vector().get_local() == w.sub(2, deepcopy=True).vector().get_local())

    assign(u1, w.sub(1))
    assert np.all(u1.vector().get_local() == w.sub(1, deepcopy=True).vector().get_local())

    assigner = FunctionAssigner(WW.sub(0), W)
    assigner.assign(ww.sub(0), w)

    assign(wr.sub(0), w)
    assert np.all(wr.sub(0, deepcopy=True).vector().get_local() == w.vector().get_local())

    assign(wr.sub(1), r)
    assert np.all(wr.sub(1, deepcopy=True).vector().get_local() == r.vector().get_local())

    assign(qqv.sub(0).sub(0), q)
    assert np.all(qqv.sub(0).sub(0, deepcopy=True).vector().get_local() == q.vector().get_local())

    with pytest.raises(RuntimeError):
        assign(qqv.sub(0), q)
    with pytest.raises(RuntimeError):
        assign(qqv.sub(1), q)
    with pytest.raises(RuntimeError):
        assign(wrr.sub(1), w)


def test_N_1_assigner(u0, u1, u2, qq, qqv, rr, w, wrr, r, W, V):

    vv = Function(W)
    assigner = FunctionAssigner(W, [V,V,V])
    assigner.assign(vv, [u0, u1, u2])

    assert np.all(vv.sub(0, deepcopy=True).vector().get_local() == u0.vector().get_local())
    assert np.all(vv.sub(1, deepcopy=True).vector().get_local() == u1.vector().get_local())
    assert np.all(vv.sub(2, deepcopy=True).vector().get_local() == u2.vector().get_local())

    assign(qqv, [qq, u1])
    assert np.all(qqv.sub(0, deepcopy=True).vector().get_local() == qq.vector().get_local())
    assert np.all(qqv.sub(1, deepcopy=True).vector().get_local() == u1.vector().get_local())

    assign(wrr, [w, rr])
    assert np.all(wrr.sub(0, deepcopy=True).vector().get_local() == w.vector().get_local())
    assert np.all(wrr.sub(1, deepcopy=True).vector().get_local() == rr.vector().get_local())

    with pytest.raises(RuntimeError):
        assign(qqv, [qq, u1, u1])

    with pytest.raises(RuntimeError):
        assign(wrr, [w, r, r])

def test_1_N_assigner(u0, u1, u2, w, qq, qqv, V, W):

    assigner = FunctionAssigner([V,V,V], W)
    assigner.assign([u0, u1, u2], w)

    assert np.all(w.sub(0, deepcopy=True).vector().get_local() == u0.vector().get_local())
    assert np.all(w.sub(1, deepcopy=True).vector().get_local() == u1.vector().get_local())
    assert np.all(w.sub(2, deepcopy=True).vector().get_local() == u2.vector().get_local())

    assign([qq, u1], qqv)

    assert np.all(qqv.sub(0, deepcopy=True).vector().get_local() == qq.vector().get_local())
    assert np.all(qqv.sub(1, deepcopy=True).vector().get_local() == u1.vector().get_local())



# -*- coding: utf-8 -*-

# Copyright (C) 2011 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import dolfin.cpp as cpp
from dolfin.fem.form import Form

__all__ = ["LocalSolver"]


class LocalSolver(cpp.fem.LocalSolver):

    def __init__(self, a, L=None, solver_type=cpp.fem.LocalSolver.SolverType.LU):
        """Create a local (cell-wise) solver for a linear variational problem
        a(u, v) = L(v).

        """

        # Store input UFL forms and solution Function
        self.a_ufl = a
        self.L_ufl = L

        # Wrap as DOLFIN forms
        a = Form(a)
        if L is None:
            # Initialize C++ base class
            cpp.fem.LocalSolver.__init__(self, a, solver_type)
        else:
            if L.empty():
                L = cpp.fem.Form(1, 0)
            else:
                L = Form(L)

        # Initialize C++ base class
        cpp.fem.LocalSolver.__init__(self, a, L, solver_type)



"""Unit tests for the HDF5 io library"""

# Copyright (C) 2012 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Chris Richardson 2013

import pytest
import os
from dolfin import *
from dolfin_utils.test import skip_if_not_HDF5, fixture, tempdir, xfail_with_serial_hdf5_in_parallel

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_parallel(tempdir):
    filename = os.path.join(tempdir, "y.h5")
    have_parallel = has_hdf5_parallel()
    hdf5 = HDF5File(MPI.comm_world, filename, "w")

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_vector(tempdir):
    filename = os.path.join(tempdir, "x.h5")
    x = Vector(MPI.comm_world, 305)
    x[:] = 1.0
    with HDF5File(x.mpi_comm(), filename, "w") as vector_file:
        vector_file.write(x, "/my_vector")

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_and_read_vector(tempdir):
    filename = os.path.join(tempdir, "vector.h5")

    # Write to file
    x = Vector(MPI.comm_world, 305)
    x[:] = 1.2
    with HDF5File(x.mpi_comm(), filename, "w") as vector_file:
        vector_file.write(x, "/my_vector")

    # Read from file
    y = Vector()
    with HDF5File(x.mpi_comm(), filename, "r") as vector_file:
        vector_file.read(y, "/my_vector", False)
        assert y.size() == x.size()
        assert (x - y).norm("l1") == 0.0

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_and_read_meshfunction_2D(tempdir):
    filename = os.path.join(tempdir, "meshfn-2d.h5")

    # Write to file
    mesh = UnitSquareMesh(20, 20)
    with HDF5File(mesh.mpi_comm(), filename, "w") as mf_file:

        # save meshfuns to compare when reading back
        meshfunctions = []
        for i in range(0,3):
            mf = MeshFunction('double', mesh, i)
            # NB choose a value to set which will be the same
            # on every process for each entity
            for cell in entities(mesh, i):
                mf[cell] = cell.midpoint()[0]
            meshfunctions.append(mf)
            mf_file.write(mf, "/meshfunction/meshfun%d" % i)

    # Read back from file
    with HDF5File(mesh.mpi_comm(), filename, "r") as mf_file:
        for i in range(0,3):
            mf2 = MeshFunction('double', mesh, i)
            mf_file.read(mf2, "/meshfunction/meshfun%d" % i)
            for cell in entities(mesh, i):
                assert meshfunctions[i][cell] == mf2[cell]

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_and_read_meshfunction_3D(tempdir):
    filename = os.path.join(tempdir, "meshfn-3d.h5")

    # Write to file
    mesh = UnitCubeMesh(10, 10, 10)
    mf_file = HDF5File(mesh.mpi_comm(), filename, "w")

    # save meshfuns to compare when reading back
    meshfunctions = []
    for i in range(0,4):
        mf = MeshFunction('double', mesh, i)
        # NB choose a value to set which will be the same
        # on every process for each entity
        for cell in entities(mesh, i):
            mf[cell] = cell.midpoint()[0]
        meshfunctions.append(mf)
        mf_file.write(mf, "/meshfunction/group/%d/meshfun"%i)
    mf_file.close()

    # Read back from file
    mf_file = HDF5File(mesh.mpi_comm(), filename, "r")
    for i in range(0,4):
        mf2 = MeshFunction('double', mesh, i)
        mf_file.read(mf2, "/meshfunction/group/%d/meshfun"%i)
        for cell in entities(mesh, i):
            assert meshfunctions[i][cell] == mf2[cell]
    mf_file.close()

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_and_read_mesh_value_collection(tempdir):
    ndiv = 5
    filename = os.path.join(tempdir, "mesh_value_collection.h5")
    mesh = UnitCubeMesh(ndiv, ndiv, ndiv)

    point2list = lambda p : [ p.x(), p.y(), p.z() ]

    # write to file
    with HDF5File(mesh.mpi_comm(), filename, 'w') as f :
        for dim in range(mesh.topology().dim()) :
            mvc = MeshValueCollection("size_t", mesh, dim)
            mesh.init(dim)
            for e in entities(mesh, dim) :
                # this can be easily computed to the check the value
                val = int(ndiv*sum(point2list(e.midpoint()))) + 1
                mvc.set_value(e.index(), val)
            f.write(mvc, "/mesh_value_collection_{}".format(dim))

    # read from file
    with HDF5File(mesh.mpi_comm(), filename, 'r') as f :
        for dim in range(mesh.topology().dim()) :
            mvc = MeshValueCollection("size_t", mesh, dim)
            f.read(mvc, "/mesh_value_collection_{}".format(dim))
            # check the values
            for (cell, lidx), val in mvc.values().items() :
                eidx = Cell(mesh, cell).entities(dim)[lidx]
                mid = point2list(MeshEntity(mesh, dim, eidx).midpoint())
                assert val == int(ndiv*sum(mid)) + 1

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_and_read_mesh_value_collection_with_only_one_marked_entity(tempdir):
    ndiv = 5
    filename = os.path.join(tempdir, "mesh_value_collection.h5")
    mesh = UnitCubeMesh(ndiv, ndiv, ndiv)
    mvc = MeshValueCollection("size_t", mesh, 3)
    mesh.init(3)
    if MPI.rank(mesh.mpi_comm()) == 0:
        mvc.set_value(0, 1)

    # write to file
    with HDF5File(mesh.mpi_comm(), filename, 'w') as f :
        f.write(mvc, "/mesh_value_collection")

    # read from file
    with HDF5File(mesh.mpi_comm(), filename, 'r') as f :
        mvc = MeshValueCollection("size_t", mesh, 3)
        f.read(mvc, "/mesh_value_collection")
        assert MPI.sum(mesh.mpi_comm(), mvc.size()) == 1
        if MPI.rank(mesh.mpi_comm()) == 0:
            assert mvc.get_value(0, 0) == 1

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_and_read_function(tempdir):
    filename = os.path.join(tempdir, "function.h5")

    mesh = UnitSquareMesh(10, 10)
    Q = FunctionSpace(mesh, "CG", 3)
    F0 = Function(Q)
    F1 = Function(Q)
    E = Expression("x[0]", degree=1)
    F0.interpolate(E)

    # Save to HDF5 File

    hdf5_file = HDF5File(mesh.mpi_comm(), filename, "w")
    hdf5_file.write(F0, "/function")
    hdf5_file.close()

    #Read back from file
    hdf5_file = HDF5File(mesh.mpi_comm(), filename, "r")
    hdf5_file.read(F1, "/function")
    result = F0.vector() - F1.vector()
    assert len(result.get_local().nonzero()[0]) == 0
    hdf5_file.close()

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_and_read_mesh_2D(tempdir):
    filename = os.path.join(tempdir, "mesh2d.h5")

    # Write to file
    mesh0 = UnitSquareMesh(20, 20)
    mesh_file = HDF5File(mesh0.mpi_comm(), filename, "w")
    mesh_file.write(mesh0, "/my_mesh")
    mesh_file.close()

    # Read from file
    mesh1 = Mesh()
    mesh_file = HDF5File(mesh0.mpi_comm(), filename, "r")
    mesh_file.read(mesh1, "/my_mesh", False)
    mesh_file.close()

    assert mesh0.num_entities_global(0) == mesh1.num_entities_global(0)
    dim = mesh0.topology().dim()
    assert mesh0.num_entities_global(dim) == mesh1.num_entities_global(dim)

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_save_and_read_mesh_3D(tempdir):
    filename = os.path.join(tempdir, "mesh3d.h5")

    # Write to file
    mesh0 = UnitCubeMesh(10, 10, 10)
    mesh_file = HDF5File(mesh0.mpi_comm(), filename, "w")
    mesh_file.write(mesh0, "/my_mesh")
    mesh_file.close()

    # Read from file
    mesh1 = Mesh()
    mesh_file = HDF5File(mesh0.mpi_comm(), filename, "r")
    mesh_file.read(mesh1, "/my_mesh", False)
    mesh_file.close()

    assert mesh0.num_entities_global(0) == mesh1.num_entities_global(0)
    dim = mesh0.topology().dim()
    assert mesh0.num_entities_global(dim) == mesh1.num_entities_global(dim)

@skip_if_not_HDF5
@xfail_with_serial_hdf5_in_parallel
def test_mpi_atomicity(tempdir):
    comm_world = MPI.comm_world
    if MPI.size(comm_world) > 1:
        filename = os.path.join(tempdir, "mpiatomic.h5")
        with HDF5File(MPI.comm_world, filename, "w") as f:
            assert f.get_mpi_atomicity() is False
            f.set_mpi_atomicity(True)
            assert f.get_mpi_atomicity() is True



# -*- coding: utf-8 -*-

# Copyright (C) 2018 Jørgen S. Dokken
#
# Distributed under the terms of the GNU Lesser Public License (LGPL),
# either version 3 of the License, or (at your option) any later
# version.

import ufl
import dolfin.cpp as cpp
from dolfin.function.functionspace import FunctionSpace


class MultiMeshFunctionSpace(object):
    def __init__(self, *args, **kwargs):
        """Create multimesh finite element function space.

        *Arguments*
        multimesh
        a :py:class:`MultiMesh <dolfin.cpp.mesh.MultiMesh>`.
        family
        a string specifying the element family,
            see :py:class:`FunctionSpace
        <dolfin.functions.functionspace.FunctionSpace>`
        for alternatives.

        This argument may also be a `FiniteElement`, in
        which case the `degree` argument should not be
        specified.
        degree
        the degree of the element.

        *Example of usage*

        .. code-block:: python

        V = MultiMeshFunctionSpace(mesh, "CG", 1)

        element = FiniteElement("Lagrange", triangle, 1)
        V = MultiMeshFunctionSpace(mesh, element)
        """
        if len(args) == 2:
            self.__init_from_ufl(*args, **kwargs)
        elif len(args) == 3:
            self._init_convenience(*args, **kwargs)
        else:
            raise NotImplementedError

    def __init_from_ufl(self, multimesh, element):
        self.info = [element]
        if not isinstance(element, ufl.FiniteElementBase):
            cpp.dolfin_error("multimeshfunctionspace.py",
                             "create function space",
                             "Illegal argument, not a finite element: " + str(element))
        # Create and add individual function spaces
        V = cpp.function.MultiMeshFunctionSpace(multimesh)
        V_parts = []
        for part in range(multimesh.num_parts()):
            V_part = FunctionSpace(multimesh.part(part), element)
            V_parts.append(V_part)
            V.add(V_part)

        # Build multimesh function space
        V.build()

        # Store full function spaces
        self._parts = V_parts
        self._cpp_object = V

    def _init_convenience(self, multimesh, family, degree):
        # Check arguments
        self.info = [family, degree]
        if not isinstance(family, str):
            cpp.dolfin_error("multimeshfunctionspace.py",
                             "create function space",
                             "Illegal argument for finite element family, not a string: " + str(family))
        if not isinstance(degree, int):
            cpp.dolfin_error("multimeshfunctionspace.py",
                             "create function space",
                             "Illegal argument for degree, not an integer: " + str(degree))
        if not isinstance(multimesh, cpp.mesh.MultiMesh):
            cpp.dolfin_error("functionspace.py",
                             "create multimesh function space",
                             "Illegal argument, not a multimesh: " + str(multimesh))

        # Create UFL element
        mesh = multimesh.part(0)
        element = ufl.FiniteElement(family, mesh.ufl_cell(), degree)

        # Create and add individual function spaces
        V = cpp.function.MultiMeshFunctionSpace(multimesh)
        V_parts = []
        for part in range(multimesh.num_parts()):
            V_part = FunctionSpace(multimesh.part(part), element)
            V_parts.append(V_part)
            V.add(V_part)

        # Build multimesh function space
        V.build()

        # Store full function spaces
        self._parts = V_parts
        self._cpp_object = V

    # Return dimension of the multimesh function space
    def dim(self):
        return self._cpp_object.dim()

    # Return the corresponding multimesh
    def multimesh(self):
        return self._cpp_object.multimesh()

    # Return dofmap for the mulitmesh function space
    def dofmap(self):
        return self._cpp_object.dofmap()

    # Return the number of function spaces (parts) of the multimesh function space
    def num_parts(self):
        return self._cpp_object.num_parts()

    # Return function space (part) number i
    def part(self, i):
        return self._cpp_object.part(i)

    # Return a common view of the multimesh function spaces (dofs global to
    # the collection of parts).
    def view(self, i):
        return self._cpp_object.view(i)

    # Add a functionspace
    def add(self, function_space):
        self._cpp_object.add(function_space.cpp_object)

    def lock_inactive_dofs(self, A, b):
        self._cpp_object.lock_inactive_dofs(A, b)


def MultiMeshVectorFunctionSpace(multimesh, family, degree, dim=None,
                                 form_degree=None):
    """Create finite element MultiMesh function space."""

    # Create UFL element
    mesh = multimesh.part(0)
    element = ufl.VectorElement(family, mesh.ufl_cell(), degree,
                                form_degree=form_degree, dim=dim)

    # Return (Py)DOLFIN FunctionSpace
    return MultiMeshFunctionSpace(multimesh, element)


def MultiMeshTensorFunctionSpace(multimesh, family, degree, shape=None,
                                 symmetry=None):
    """Create finite element MultiMesh function space."""

    # Create UFL element
    mesh = multimesh.part(0)
    element = ufl.TensorElement(family, mesh.ufl_cell(), degree,
                                shape, symmetry)

    # Return (Py)DOLFIN FunctionSpace
    return MultiMeshFunctionSpace(mesh, element)



"""Unit tests for the fem interface"""

# Copyright (C) 2011-2014 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Marie E. Rognes (meg@simula.no), 2012
# Modified by Martin S. Alnaes (martinal@simula.no), 2014

import pytest
import numpy
import ufl
from dolfin import *
from dolfin_utils.test import skip_in_parallel, fixture


@fixture
def square():
    return UnitSquareMesh(8, 8)


@fixture
def square_boundary(square):
    return BoundaryMesh(square, "exterior")


@fixture
def cube_boundary(cube):
    return BoundaryMesh(cube, "exterior")


@fixture
def cube():
    return UnitCubeMesh(2, 2, 2)


@fixture
def V1(square_boundary):
    return FunctionSpace(square_boundary, "CG", 1)


@fixture
def VV1(square_boundary):
    return VectorFunctionSpace(square_boundary, "CG", 1)


@fixture
def Q1(square_boundary):
    return FunctionSpace(square_boundary, "DG", 0)


@fixture
def V2(cube_boundary):
    return FunctionSpace(cube_boundary, "CG", 1)


@fixture
def VV2(cube_boundary):
    return VectorFunctionSpace(cube_boundary, "CG", 1)


@fixture
def Q2(cube_boundary):
    return FunctionSpace(cube_boundary, "DG", 0)


def test_assemble_functional(V1, V2):

    mesh = V1.mesh()
    surfacearea = assemble(1*dx(mesh))
    assert round(surfacearea - 4.0, 7) == 0

    u = Function(V1)
    u.vector()[:] = 1.0
    surfacearea = assemble(u*dx)
    assert round(surfacearea - 4.0, 7) == 0

    u = Function(V2)
    u.vector()[:] = 1.0
    surfacearea = assemble(u*dx)
    assert round(surfacearea - 6.0, 7) == 0

    f = Expression("1.0", degree=0)
    u = interpolate(f, V1)
    surfacearea = assemble(u*dx)
    assert round(surfacearea - 4.0, 7) == 0

    f = Expression("1.0", degree=0)
    u = interpolate(f, V2)
    surfacearea = assemble(u*dx)
    assert round(surfacearea - 6.0, 7) == 0


def test_assemble_linear(V1, Q1, square_boundary, V2, Q2, cube_boundary):

    u = Function(V1)
    w = TestFunction(Q1)
    u.vector()[:] = 0.5
    facetareas = MPI.sum(square_boundary.mpi_comm(),
                         assemble(u*w*dx).get_local().sum())
    assert round(facetareas - 2.0, 7) == 0

    u = Function(V2)
    w = TestFunction(Q2)
    u.vector()[:] = 0.5
    a = u*w*dx
    b = assemble(a)
    facetareas = MPI.sum(cube_boundary.mpi_comm(),
                         assemble(u*w*dx).get_local().sum())
    assert round(facetareas - 3.0, 7) == 0

    mesh = UnitSquareMesh(8, 8)
    bdry = BoundaryMesh(mesh, "exterior")
    V = FunctionSpace(mesh, "CG", 1)
    u = TrialFunction(V)
    v = TestFunction(V)

    BV = FunctionSpace(bdry, "CG", 1)
    bu = TrialFunction(BV)
    bv = TestFunction(BV)

    a = MPI.sum(mesh.mpi_comm(),
                assemble(inner(u, v)*ds).array().sum())
    b = MPI.sum(bdry.mpi_comm(),
                assemble(inner(bu, bv)*dx).array().sum())
    assert round(a - b, 7) == 0


@skip_in_parallel
def test_assemble_bilinear_1D_2D(square, V1, square_boundary):

    V = FunctionSpace(square, 'CG', 1)
    u = TrialFunction(V)
    v = TestFunction(V)

    bu = TrialFunction(V1)
    bv = TestFunction(V1)

    a = MPI.sum(square.mpi_comm(),
                assemble(inner(u, v)*ds).array().sum())
    b = MPI.sum(square_boundary.mpi_comm(),
                assemble(inner(bu, bv)*dx).array().sum())
    assert round(a - b, 7) == 0

    # Assemble over subset of mesh facets
    subdomain = CompiledSubDomain("near(x[1], 0.0)")
    bottom = MeshFunction("size_t", square, square.topology().dim()-1)
    bottom.set_all(0)
    subdomain.mark(bottom, 1)
    dss = ds(subdomain_data=bottom)
    foo = MPI.sum(square.mpi_comm(),
                  abs(assemble(inner(grad(u)[0],
                                     grad(v)[0])*dss(1)).array()).sum())

    # Assemble over all cells of submesh created from subset of
    # boundary mesh
    bottom2 = MeshFunction("size_t", square_boundary, square_boundary.topology().dim())
    bottom2.set_all(0)
    subdomain.mark(bottom2, 1)
    BV = FunctionSpace(SubMesh(square_boundary, bottom2, 1), "CG", 1)
    bu = TrialFunction(BV)
    bv = TestFunction(BV)
    bar = MPI.sum(square_boundary.mpi_comm(),
                  abs(assemble(inner(grad(bu)[0],
                                     grad(bv)[0])*dx).array()).sum())
    # Should give same result
    assert round(bar - foo, 7) == 0


@skip_in_parallel
def test_assemble_bilinear_2D_3D(cube, V2, cube_boundary):

    V = FunctionSpace(cube, 'CG', 1)
    u = TrialFunction(V)
    v = TestFunction(V)

    # V2 is a FunctionSpace over cube_boundary
    bu = TrialFunction(V2)
    bv = TestFunction(V2)

    a = MPI.sum(cube.mpi_comm(),
                assemble(inner(u, v)*ds).array().sum())
    b = MPI.sum(cube_boundary.mpi_comm(),
                assemble(inner(bu, bv)*dx).array().sum())
    assert round(a - b, 7) == 0

    # Assemble over subset of mesh facets
    subdomain = CompiledSubDomain("near(x[1], 0.0)")
    bottom = MeshFunction("size_t", cube, cube.topology().dim()-1)
    bottom.set_all(0)
    subdomain.mark(bottom, 1)
    dss = ds(subdomain_data=bottom)
    foo = MPI.sum(cube.mpi_comm(),
                  abs(assemble(inner(grad(u)[0],
                                     grad(v)[0])*dss(1)).array()).sum())

    # Assemble over all cells of submesh created from subset of
    # boundary mesh
    bottom2 = MeshFunction("size_t", cube_boundary, cube_boundary.topology().dim())
    bottom2.set_all(0)
    subdomain.mark(bottom2, 1)
    BV = FunctionSpace(SubMesh(cube_boundary, bottom2, 1), "CG", 1)
    bu = TrialFunction(BV)
    bv = TestFunction(BV)
    bar = MPI.sum(cube_boundary.mpi_comm(),
                  abs(assemble(inner(grad(bu)[0],
                                     grad(bv)[0])*dx).array()).sum())

    # Should give same result
    assert round(bar - foo, 7) == 0


@fixture
def base():
    n = 16
    plane = CompiledSubDomain("near(x[1], 1.0)")
    square = UnitSquareMesh(n, n)

    square3d = SubMesh(BoundaryMesh(UnitCubeMesh(n, n, n), "exterior"), plane)
    global_normal = Expression(("0.0", "1.0", "0.0"), degree=0)
    square3d.init_cell_orientations(global_normal)

    RT2 = FiniteElement("RT", square.ufl_cell(), 1)
    RT3 = FiniteElement("RT", square3d.ufl_cell(), 1)
    DG2 = FiniteElement("DG", square.ufl_cell(), 0)
    DG3 = FiniteElement("DG", square3d.ufl_cell(), 0)

    return [(RT2, RT3), (DG2, DG3), (square, square3d)]


@fixture
def RT2(base):
    return FunctionSpace(base[2][0], base[0][0])


@fixture
def RT3(base):
    return FunctionSpace(base[2][1], base[0][1])


@fixture
def W2(base):
    """ RT2 * DG2 """
    return FunctionSpace(base[2][0], base[0][0] * base[1][0])


@fixture
def W3(base):
    """ RT3 * DG3 """
    return FunctionSpace(base[2][1], base[0][1] * base[1][1])


@fixture
def QQ2(base):
    """ DG2 * DG2 """
    return FunctionSpace(base[2][0], base[1][0] * base[1][0])


@fixture
def QQ3(base):
    """ DG3 * DG3 """
    return FunctionSpace(base[2][1], base[1][1] * base[1][1])


@skip_in_parallel
def test_basic_rt(RT2, RT3):

    f2 = Expression(("2.0", "1.0"), degree=0)
    f3 = Expression(("1.0", "0.0", "2.0"), degree=0)

    u2 = TrialFunction(RT2)
    u3 = TrialFunction(RT3)
    v2 = TestFunction(RT2)
    v3 = TestFunction(RT3)

    # Project
    pw2 = project(f2, RT2)
    pw3 = project(f3, RT3)
    pa2 = assemble(pw2**2*dx)
    pa3 = assemble(pw3**2*dx)

    # Project explicitly
    a2 = inner(u2, v2)*dx
    a3 = inner(u3, v3)*dx
    L2 = inner(f2, v2)*dx
    L3 = inner(f3, v3)*dx
    w2 = Function(RT2)
    w3 = Function(RT3)
    A2 = assemble(a2)
    b2 = assemble(L2)
    A3 = assemble(a3)
    b3 = assemble(L3)
    solve(A2, w2.vector(), b2)
    solve(A3, w3.vector(), b3)
    a2 = assemble(w2**2*dx)
    a3 = assemble(w3**2*dx)

    # Compare various results
    assert round((w2.vector() - pw2.vector()).norm("l2"), 4) == 0
    assert round((w3.vector() - pw3.vector()).norm("l2"), 4) == 0
    # 2d
    assert round(a2 - 5.0, 7) == 0
    assert round(pa2 - 5.0, 7) == 0
    # 3d
    assert round(a3 - 5.0, 7) == 0
    assert round(pa3 - 5.0, 6) == 0


@skip_in_parallel
def test_mixed_poisson_solve(W2, W3):

    f = Constant(1.0)

    # Solve mixed Poisson on standard unit square
    (sigma2, u2) = TrialFunctions(W2)
    (tau2, v2) = TestFunctions(W2)
    a = (inner(sigma2, tau2) + div(tau2)*u2 + div(sigma2)*v2)*dx
    L = f*v2*dx
    w2 = Function(W2)
    solve(a == L, w2)

    # Solve mixed Poisson on unit square in 3D
    (sigma3, u3) = TrialFunctions(W3)
    (tau3, v3) = TestFunctions(W3)
    a = (inner(sigma3, tau3) + div(tau3)*u3 + div(sigma3)*v3)*dx
    L = f*v3*dx
    w3 = Function(W3)
    solve(a == L, w3)

    # Check that results are about the same
    assert round(assemble(inner(w2, w2)*dx) - assemble(inner(w3, w3)*dx)) == 0


@fixture
def m():
    return 3


@fixture
def cube(m):
    return UnitCubeMesh(m, m, m)


@fixture
def cube_boundary(cube):
    return BoundaryMesh(cube, "exterior")


@fixture
def plane():
    return CompiledSubDomain("near(x[1], 0.0)")


@fixture
def square_boundary_(m):
    square = UnitSquareMesh(m, m)
    return BoundaryMesh(square, "exterior")


@fixture
def line():
    return CompiledSubDomain("near(x[0], 0.0)")


@fixture
def mesh3(cube_boundary, plane):
    return BoundaryMesh(SubMesh(cube_boundary, plane), "exterior")


@fixture
def bottom1(square_boundary_, plane):
    return SubMesh(square_boundary_, plane)


@fixture
def bottom2(cube_boundary, plane):
    return SubMesh(cube_boundary, plane)


@fixture
def bottom3(mesh3, line):
    return SubMesh(mesh3, line)


@skip_in_parallel
def test_normals_2D_1D(bottom1, m):
    "Testing assembly of normals for 1D meshes embedded in 2D"

    n = FacetNormal(bottom1)
    a = inner(n, n)*ds
    value_bottom1 = assemble(a)
    assert round(value_bottom1 - 2.0, 7) == 0
    b = inner(n('+'), n('+'))*dS
    b1 = assemble(b)
    c = inner(n('+'), n('-'))*dS
    c1 = assemble(c)
    assert round(b1 - m + 1, 7) == 0
    assert round(c1 + b1, 7) == 0


@skip_in_parallel
def test_normals_3D_1D(bottom3, m):
    "Testing assembly of normals for 1D meshes embedded in 3D"

    n = FacetNormal(bottom3)
    a = inner(n, n)*ds
    v1 = assemble(a)
    assert round(v1 - 2.0, 7) == 0
    b = inner(n('+'), n('+'))*dS
    b1 = assemble(b)
    c = inner(n('+'), n('-'))*dS
    c1 = assemble(c)
    assert round(b1 - m + 1, 7) == 0
    assert round(c1 + b1, 7) == 0


@skip_in_parallel
def test_normals_3D_2D(bottom2):
    "Testing assembly of normals for 2D meshes embedded in 3D"

    n = FacetNormal(bottom2)
    a = inner(n, n)*ds
    v1 = assemble(a)
    assert round(v1 - 4.0, 7) == 0

    b = inner(n('+'), n('+'))*dS
    b1 = assemble(b)
    c = inner(n('+'), n('-'))*dS
    c1 = assemble(c)
    assert round(c1 + b1, 7) == 0


@skip_in_parallel
def test_cell_volume(m, bottom1, bottom2, bottom3):
    "Testing assembly of volume for embedded meshes"

    volume = CellVolume(bottom1)
    a = volume*dx
    b = assemble(a)
    assert round(b - 1.0/m, 7) == 0

    volume = CellVolume(bottom3)
    a = volume*dx
    b = assemble(a)
    assert round(b - 1.0/m, 7) == 0

    volume = CellVolume(bottom2)
    a = volume*dx
    b = assemble(a)
    assert round(b - 1.0/(2*m*m), 7) == 0


@skip_in_parallel
def test_circumradius(m, bottom1, bottom2, bottom3):
    "Testing assembly of circumradius for embedded meshes"

    r = Circumradius(bottom1)
    a = r*dx
    b = assemble(a)
    assert round(b - 0.5*(1.0/m), 7) == 0

    r = Circumradius(bottom3)
    a = r*dx
    b = assemble(a)
    assert round(b - 0.5*(1.0/m), 7) == 0

    square = UnitSquareMesh(m, m)
    r = Circumradius(square)
    a = r*dx
    b0 = assemble(a)

    r = Circumradius(bottom2)
    a = r*dx
    b1 = assemble(a)
    assert round(b0 - b1, 7) == 0


@skip_in_parallel
def test_facetarea(bottom1, bottom2, bottom3, m):
    "Testing assembly of facet area for embedded meshes"

    area = FacetArea(bottom1)
    a = area*ds
    b = assemble(a)
    assert round(b - 2.0, 7) == 0

    area = FacetArea(bottom3)
    a = area*ds
    b = assemble(a)
    assert round(b - 2.0, 7) == 0

    square = UnitSquareMesh(m, m)
    area = FacetArea(square)
    a = area*ds
    b0 = assemble(a)

    area = FacetArea(bottom2)
    a = area*ds
    b1 = assemble(a)
    assert round(b0 - b1, 7) == 0


@skip_in_parallel
def test_derivative(QQ2, QQ3):
    for W in [QQ2, QQ3]:
        w = Function(W)
        dim = w.value_dimension(0)
        w.interpolate(Constant([42.0*(i+1) for i in range(dim)]))

        # Derivative w.r.t. mixed space
        u, v = split(w)
        F = u*v*dx
        dF = derivative(F, w)
        b1 = assemble(dF)


def test_coefficient_derivatives(V1, V2):
    for V in [V1, V2]:
        f = Function(V)
        g = Function(V)
        v = TestFunction(V)
        u = TrialFunction(V)

        f.interpolate(Expression("1.0 + x[0] + x[1]", degree=1))
        g.interpolate(Expression("2.0 + x[0] + x[1]", degree=1))

        # Since g = f + 1, define dg/df = 1
        cd = {g: 1}

        # Handle relation between g and f in derivative
        M = g**2*dx
        L = derivative(M, f, v, coefficient_derivatives=cd)
        a = derivative(L, f, u, coefficient_derivatives=cd)
        A0 = assemble(a).norm('frobenius')
        b0 = assemble(L).norm('l2')

        # Manually construct the above case
        M = g**2*dx
        L = 2*g*v*dx
        a = 2*u*v*dx
        A1 = assemble(a).norm('frobenius')
        b1 = assemble(L).norm('l2')

        # Compare
        assert round(A0 - A1, 7) == 0.0
        assert round(b0 - b1, 7) == 0.0



from dolfin import *
set_log_level(LogLevel.ERROR)

# Create mesh and define function space
mesh = UnitSquareMesh(32, 32)

V = FunctionSpace(mesh, "Lagrange", 1)

marker = MeshFunction("size_t", mesh, mesh.topology().dim(), 0)
for c in cells(mesh):
    marker[c] = c.midpoint().y() < 0.5
    #marker[c] = c.midpoint().x() < 0.5

submesh1 = MeshView.create(marker, 1)
submesh2 = MeshView.create(marker, 0)

V1 = FunctionSpace(submesh1, "Lagrange", 1)
V2 = FunctionSpace(submesh2, "Lagrange", 1)

# Define Dirichlet boundary (x = 0 or x = 1)
def boundary(x):
    return x[1] < DOLFIN_EPS or x[1] > 1.0 - DOLFIN_EPS
    #return x[0] < DOLFIN_EPS or x[0] > 1.0 - DOLFIN_EPS

def boundarySub1(x):
    return x[1] < DOLFIN_EPS
    #return x[0] < DOLFIN_EPS

def boundarySub2(x):
    return x[1] > 1.0 - DOLFIN_EPS
    #return x[0] > 1.0 - DOLFIN_EPS

# Define boundary condition
u0 = Constant(0.0)
# Whole domain
bc = DirichletBC(V, u0, boundary)
# Subdomain 1
bc1 = DirichletBC(V1, u0, boundarySub1)
# Subdomain 2
bc2 = DirichletBC(V2, u0, boundarySub2)

# Define variational problem
# Whole domain
u = TrialFunction(V)
v = TestFunction(V)
# Subdomain 1
u1 = TrialFunction(V1)
v1 = TestFunction(V1)
# Subdomain 2
u2 = TrialFunction(V2)
v2 = TestFunction(V2)

f = Expression("10*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)", degree=2)

# Whole domain
a = inner(grad(u), grad(v))*dx
L = f*v*dx
u = Function(V)
solve(a == L, u, bc)
# Subdomain 1
a1 = inner(grad(u1), grad(v1))*dx
L1 = f*v1*dx
u1 = Function(V1)
solve(a1 == L1, u1, bc1)
# Subdomain 2
a2 = inner(grad(u2), grad(v2))*dx
L2 = f*v2*dx
u2 = Function(V2)
solve(a2 == L2, u2, bc2)

## Export result
encoding = XDMFFile.Encoding.HDF5 if has_hdf5() else XDMFFile.Encoding.ASCII

out_global = XDMFFile(MPI.comm_world, "meshview-mapping-2D2D-global.xdmf")
out_sub1 = XDMFFile(MPI.comm_world, "meshview-mapping-2D2D-subdomain1.xdmf")
out_sub2 = XDMFFile(MPI.comm_world, "meshview-mapping-2D2D-subdomain2.xdmf")

if MPI.size(MPI.comm_world) > 1 and encoding == XDMFFile.Encoding.ASCII:
    print("XDMF file output not supported in parallel without HDF5")
else:
    out_global.write(u, encoding)
    out_sub1.write(u1, encoding)
    out_sub2.write(u2, encoding)




"""Unit tests for the Edge class"""

# Copyright (C) 2011 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2011-02-26
# Last changed: 2014-05-30

import pytest
from dolfin import *
from dolfin_utils.test import fixture, skip_in_parallel


@fixture
def cube():
    return UnitCubeMesh(5, 5, 5)


@fixture
def square():
    return UnitSquareMesh(5, 5)


@pytest.fixture(scope='module', params=range(2))
def meshes(cube, square, request):
    mesh = [cube, square]
    return mesh[request.param]


@skip_in_parallel
def test_2DEdgeLength(square):
    """Iterate over edges and sum length."""
    length = 0.0
    for e in edges(square):
        length += e.length()
    assert round(length - 19.07106781186544708362, 7) == 0


@skip_in_parallel
def test_3DEdgeLength(cube):
    """Iterate over edges and sum length."""
    length = 0.0
    for e in edges(cube):
        length += e.length()
    assert round(length - 278.58049080280125053832, 7) == 0


def test_EdgeDot(meshes):
    """Iterate over edges compute dot product with ."""
    for e in edges(meshes):
        dot = e.dot(e)/(e.length()**2)
        assert round(dot - 1.0, 7) == 0




import numpy
from dolfin import *


def test_manifold_point_search():
    # Simple two-triangle surface in 3d
    vertices = [
        (0.0, 0.0, 1.0),
        (1.0, 1.0, 1.0),
        (1.0, 0.0, 0.0),
        (0.0, 1.0, 0.0),
        ]
    cells = [
        (0, 1, 2),
        (0, 1, 3),
    ]
    mesh = Mesh()
    me = MeshEditor()
    me.open(mesh, "triangle", 2, 3)
    me.init_vertices(len(vertices))
    for i, v in enumerate(vertices):
        me.add_vertex(i, numpy.array(v, dtype='float'))
    me.init_cells(len(cells))
    for i, c in enumerate(cells):
        me.add_cell(i, numpy.array(c, dtype='uint'))
    me.close()

    mesh.init_cell_orientations(Expression(("0.0", "0.0", "1.0"), degree=0))

    bb = mesh.bounding_box_tree()
    p = Point(0.5, 0.25, 0.75)
    assert bb.compute_first_entity_collision(p) == 0

    p = Point(0.25, 0.5, 0.75)
    assert bb.compute_first_entity_collision(p) == 1



"""Unit tests for the function library"""

# Copyright (C) 2007 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2007-05-24
# Last changed: 2011-01-28

import pytest
from numpy import array
from dolfin import *
from ufl import UFLException

def test_name_argument():
    u = Constant(1.0)
    v = Constant(1.0, name="v")
    assert u.name() == "f_%d" % u.count()
    assert v.name() == "v"
    assert str(v) == "v"


def testConstantInit():
    c0 = Constant(1.)
    c1 = Constant([2, 3], interval)
    c2 = Constant([[2, 3], [3, 4]], triangle)
    c3 = Constant(array([2, 3]), tetrahedron)
    c4 = Constant([[2, 3], [3, 4]], quadrilateral)
    c5 = Constant(array([2, 3]), hexahedron)

    # FIXME:
    assert c0.cell() is None
    assert c1.cell() == interval
    assert c2.cell() == triangle
    assert c3.cell() == tetrahedron
    assert c4.cell() == quadrilateral
    assert c5.cell() == hexahedron

    assert c0.ufl_shape == ()
    assert c1.ufl_shape == (2,)
    assert c2.ufl_shape == (2, 2)
    assert c3.ufl_shape == (2,)
    assert c4.ufl_shape == (2, 2)
    assert c5.ufl_shape == (2,)


def testGrad():
    import ufl
    zero = ufl.constantvalue.Zero((2, 3))
    c0 = Constant(1.)
    c3 = Constant(array([2, 3]), tetrahedron)

    def gradient(c):
        return grad(c)
    with pytest.raises(UFLException):
        grad(c0)
    assert zero == gradient(c3)


@pytest.mark.parametrize('mesh_factory', [(UnitCubeMesh, (8, 8, 8)), (UnitCubeMesh.create, (8, 8, 8, CellType.Type.hexahedron))])
def test_compute_vertex_values(mesh_factory):
    from numpy import zeros, all, array

    func, args = mesh_factory
    mesh = func(*args)

    e0 = Constant(1)
    e1 = Constant((1, 2, 3))

    # e0_values = zeros(mesh.num_vertices(),dtype='d')
    # e1_values = zeros(mesh.num_vertices()*3,dtype='d')

    e0_values = e0.compute_vertex_values(mesh)
    e1_values = e1.compute_vertex_values(mesh)

    assert all(e0_values == 1)
    assert all(e1_values[:mesh.num_vertices()] == 1)
    assert all(e1_values[mesh.num_vertices():mesh.num_vertices()*2] == 2)
    assert all(e1_values[mesh.num_vertices()*2:mesh.num_vertices()*3] == 3)


def test_values():
    import numpy as np

    c0 = Constant(1.)
    c0_vals = c0.values()
    assert np.all(c0_vals == np.array([1.], dtype=np.double))

    c1 = Constant((1., 2.))
    c1_vals = c1.values()
    assert np.all(c1_vals == np.array([1., 2.], dtype=np.double))

    c2 = Constant((1., 2., 3.))
    c2_vals = c2.values()
    assert np.all(c2_vals == np.array([1., 2., 3.], dtype=np.double))


def test_str():
    c0 = Constant(1.)
    c0.str(False)
    c0.str(True)

    c1 = Constant((1., 2., 3.))
    c1.str(False)
    c1.str(True)


def test_assign():
    c0 = Constant(1.)
    assert c0.values() == (1,)
    c0.assign(Constant(3))
    assert c0.values() == (3,)

    c1 = Constant([1, 2])
    assert (c1.values() == (1, 2)).all()
    c1.assign(Constant([3, 4]))
    assert (c1.values() == (3, 4)).all()



"Unit tests for SparsityPattern"

# Copyright (C) 2017 Nathan Sime
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import pytest
import numpy as np
from dolfin import *
from dolfin_utils.test import *


def count_on_and_off_diagonal_nnz(primary_codim_entries, local_range):
    nnz_on_diagonal = sum(1 for entry in primary_codim_entries
                          if local_range[0] <= entry < local_range[1])
    nnz_off_diagonal = sum(1 for entry in primary_codim_entries
                           if not local_range[0] <= entry < local_range[1])
    return nnz_on_diagonal, nnz_off_diagonal


@fixture
def mesh():
    return UnitSquareMesh(10, 10)


@fixture
def V(mesh):
    return FunctionSpace(mesh, "CG", 1)


def test_str(mesh, V):
    dm = V.dofmap()
    index_map = dm.index_map()

    # Build sparse tensor layout (for assembly of matrix)
    tl = TensorLayout(mesh.mpi_comm(), 0, TensorLayout.Sparsity.SPARSE)
    tl.init([index_map, index_map], TensorLayout.Ghosts.UNGHOSTED)
    sp = tl.sparsity_pattern()
    sp.init([index_map, index_map])
    SparsityPatternBuilder.build(sp, mesh, [dm, dm],
                                 True, False, False, False,
                                 False, init=False, finalize=True)

    sp.str(False)
    sp.str(True)


def test_insert_local(mesh, V):
    dm = V.dofmap()
    index_map = dm.index_map()

    # Build sparse tensor layout
    tl = TensorLayout(mesh.mpi_comm(), 0, TensorLayout.Sparsity.SPARSE)
    tl.init([index_map, index_map], TensorLayout.Ghosts.UNGHOSTED)
    sp = tl.sparsity_pattern()
    sp.init([index_map, index_map])

    primary_dim_entries = [0, 1, 2]
    primary_codim_entries = [0, 1, 2]
    entries = np.array([primary_dim_entries, primary_codim_entries], dtype=np.intc)
    sp.insert_local(entries)

    sp.apply()

    assert len(primary_dim_entries) * len(primary_codim_entries) == sp.num_nonzeros()

    nnz_d = sp.num_nonzeros_diagonal()
    for local_row in range(len(nnz_d)):
        assert nnz_d[local_row] == (len(primary_codim_entries) if local_row in primary_dim_entries else 0)


def test_insert_global(mesh, V):
    dm = V.dofmap()
    index_map = dm.index_map()
    local_range = index_map.local_range()

    # Build sparse tensor layout
    tl = TensorLayout(mesh.mpi_comm(), 0, TensorLayout.Sparsity.SPARSE)
    tl.init([index_map, index_map], TensorLayout.Ghosts.UNGHOSTED)
    sp = tl.sparsity_pattern()
    sp.init([index_map, index_map])

    # Primary dim (row) entries need to be local to the process, so we ensure
    # they're in the local range of the index map
    primary_dim_local_entries = np.array([0, 1, 2], dtype=np.intc)
    primary_dim_entries = primary_dim_local_entries + local_range[0]

    # The codim (column) entries will be added to the same global entries
    # on each process.
    primary_codim_entries = np.array([0, 1, 2], dtype=np.intc)
    entries = np.array([primary_dim_entries, primary_codim_entries], dtype=np.intc)

    sp.insert_global(entries)
    sp.apply()

    nnz_d = sp.num_nonzeros_diagonal()
    nnz_od = sp.num_nonzeros_off_diagonal()

    rank = MPI.rank(mesh.mpi_comm())
    size = MPI.size(mesh.mpi_comm())

    # Tabulate on diagonal and off diagonal nnzs
    nnz_on_diagonal, nnz_off_diagonal = count_on_and_off_diagonal_nnz(
        primary_codim_entries, local_range)

    # Compare tabulated and sparsity pattern nnzs
    for local_row in range(len(nnz_d)):
        if local_range[0] <= local_row < local_range[1]:
            assert nnz_d[local_row] == (nnz_on_diagonal if local_row in primary_dim_local_entries else 0)
        else:
            assert nnz_od[local_row] == (nnz_off_diagonal if local_row in primary_dim_local_entries else 0)


def test_insert_local_global(mesh, V):
    dm = V.dofmap()
    index_map = dm.index_map()
    local_range = index_map.local_range()

    # Build sparse tensor layout
    tl = TensorLayout(mesh.mpi_comm(), 0, TensorLayout.Sparsity.SPARSE)
    tl.init([index_map, index_map], TensorLayout.Ghosts.UNGHOSTED)
    sp = tl.sparsity_pattern()
    sp.init([index_map, index_map])

    # Primary dim (row) entries need to be local to the process, so we ensure
    # they're in the local range of the index map
    primary_dim_local_entries = np.array([0, 1, 2], dtype=np.intc)
    primary_dim_entries = primary_dim_local_entries

    # The codim (column) entries will be added to the same global entries
    # on each process.
    primary_codim_entries = np.array([0, 1, 2], dtype=np.intc)
    entries = np.array([primary_dim_entries, primary_codim_entries], dtype=np.intc)

    sp.insert_local_global(entries)
    sp.apply()

    nnz_d = sp.num_nonzeros_diagonal()
    nnz_od = sp.num_nonzeros_off_diagonal()

    rank = MPI.rank(mesh.mpi_comm())
    size = MPI.size(mesh.mpi_comm())

    # Tabulate on diagonal and off diagonal nnzs
    nnz_on_diagonal, nnz_off_diagonal = count_on_and_off_diagonal_nnz(
        primary_codim_entries, local_range)

    # Compare tabulated and sparsity pattern nnzs
    for local_row in range(len(nnz_d)):
        if local_range[0] <= local_row < local_range[1]:
            assert nnz_d[local_row] == (nnz_on_diagonal if local_row in primary_dim_local_entries else 0)
        else:
            assert nnz_od[local_row] == (nnz_off_diagonal if local_row in primary_dim_local_entries else 0)



import pytest
import gc
from dolfin import MPI

def pytest_runtest_teardown(item):
    """Collect garbage after every test to force calling
    destructors which might be collective"""

    # Do the normal teardown
    item.teardown()

    # Collect the garbage (call destructors collectively)
    del item
    # NOTE: How are we sure that 'item' does not hold references
    #       to temporaries and someone else does not hold a reference
    #       to 'item'?! Well, it seems that it works...
    gc.collect()
    MPI.barrier(MPI.comm_world)



"""Unit tests for timing facilities"""

# Copyright (C) 2017 Jan Blechta
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import pytest

import gc
import random
from time import sleep

from dolfin import *


# Seed random generator for determinism
random.seed(0)

def get_random_task_name():
    """Get pseudo-random string"""
    return hex(random.randint(0, 1e32))


def test_context_manager_named():
    """Test that named Timer works as context manager"""
    task = get_random_task_name()

    # Execute task in the context manager
    with Timer(task) as t:
        sleep(0.05)
        assert t.elapsed()[0] >= 0.05

    # Check timing
    t = timing(task, TimingClear.clear)
    assert t[0] == 1
    assert t[1] >= 0.05


def test_context_manager_anonymous():
    """Test that anonymous Timer works as context manager"""
    with Timer() as t:
        sleep(0.05)
        assert t.elapsed()[0] >= 0.05


# Test case for decorated free function
fun2_task = get_random_task_name()
@timed(fun2_task)
def fun2(*args, **kwargs):
    "Foo"
    sleep(0.05)
    return args, kwargs

class C(object):
    # Test case for decorated instancemethod
    task_method2 = get_random_task_name()
    @timed(task_method2)
    def method2(self, *args, **kwargs):
        "Foo"
        sleep(0.05)
        return args, kwargs

    # Test case for decorated staticmethod
    task_method3 = get_random_task_name()
    @staticmethod
    @timed(task_method3)
    def method3(*args, **kwargs):
        "Foo"
        sleep(0.05)
        return args, kwargs

    # Test case for decorated classmethod
    task_method5 = get_random_task_name()
    @classmethod
    @timed(task_method5)
    def method5(cls, *args, **kwargs):
        "Foo"
        sleep(0.05)
        return args, kwargs

# Testing object with decorated methods
o = C()


@pytest.mark.parametrize(("fun", "task"), [
    (fun2, fun2_task),
    (o.method2, o.task_method2),
    (o.method3, o.task_method3),
    (o.method5, o.task_method5),
    (C.method3, C.task_method3),
    (C.method5, C.task_method5),
])
def test_decorator_functionality(fun, task):
    """Test functionality of timed decorator on function,
    instancemethod, classmethod, staticmethod..."""
    # Check that decorator preserves docstring
    assert fun.__doc__ == "Foo"

    # Check that function call works and returns what expected
    assert fun(1, 2, 3, four=5) == ((1, 2, 3), {'four': 5})
    assert fun(1, 2, 3, four=5) == ((1, 2, 3), {'four': 5})

    # Check that correct timing was recorded
    t = timing(task, TimingClear.clear)
    assert t[0] == 2
    assert t[1] >= 0.1


def test_decorator_timer_scope():
    """Check that underlying Timer object is not launched
    by decorator unless function is invoked"""
    task = get_random_task_name()

    # Delete eventual previous timing
    try:
        timing(task, TimingClear.clear)
    except RuntimeError:
        pass

    @timed(task)
    def foo():
        pass

    # Delete function (including eventual timer which should not
    # exist yet) before using it
    del foo
    gc.collect()

    # Check that there's no timing entry
    with pytest.raises(RuntimeError):
        timing(task, TimingClear.clear)


def test_decorator_timing_correctness():
    """Check that timing is measured really when the function
    is executed, not during existence of decorator"""
    task = get_random_task_name()

    @timed(task)
    def foo():
        pass

    sleep(0.05)
    foo()
    sleep(0.05)

    # Check that sleeping above did not influence timer
    assert timing(task, TimingClear.clear)[1] < 0.1



# -*- coding: utf-8 -*-
"""This module handles the Function class in Python.

"""
# Copyright (C) 2009-2014 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import ufl
from .functionspace import FunctionSpace, MixedFunctionSpace
from .multimeshfunctionspace import MultiMeshFunctionSpace


__all__ = ["TestFunction", "TrialFunction", "Argument",
           "TestFunctions", "TrialFunctions"]

# --- Subclassing of ufl.{Basis, Trial, Test}Function ---

# TODO: Update this message to clarify dolfin.FunctionSpace vs
# ufl.FunctionSpace
_ufl_dolfin_difference_message = r"""\ When constructing an Argument, TestFunction or TrialFunction, you
must to provide a FunctionSpace and not a FiniteElement.  The
FiniteElement class provided by ufl only represents an abstract finite
element space and is only used in standalone .ufl files, while the
FunctionSpace provides a full discrete function space over a given
mesh and should be used in dolfin programs in Python.  """


class Argument(ufl.Argument):
    """UFL value: Representation of an argument to a form.

    This is the overloaded PyDOLFIN variant.
    """

    def __init__(self, V, number, part=None):

        # Check argument
        if not isinstance(V, (FunctionSpace, MultiMeshFunctionSpace)):
            if isinstance(V, (ufl.FiniteElementBase, ufl.FunctionSpace)):
                raise TypeError(_ufl_dolfin_difference_message)
            else:
                raise TypeError("Illegal argument for creation of Argument, not a FunctionSpace: " + str(V))
            raise TypeError("Illegal argument for creation of Argument, not a FunctionSpace: " + str(V))

        # Handle MultiMesh
        if isinstance(V, MultiMeshFunctionSpace):
            self._V_multi = V._cpp_object
            V = V._parts[0]

        # Initialize UFL Argument
        ufl.Argument.__init__(self, V.ufl_function_space(), number, part)

        self._V = V

    def function_space(self):
        "Return the FunctionSpace"
        return self._V

    def __eq__(self, other):
        """Extending UFL __eq__ here to distinguish test and trial functions
        in different function spaces with same ufl element.

        """
        return (isinstance(other, Argument) and
                self.number() == other.number() and
                self.part() == other.part() and
                self._V == other._V)

    def __hash__(self):
        return ufl.Argument.__hash__(self)


def TestFunction(V, part=None):
    """UFL value: Create a test function argument to a form.

    This is the overloaded PyDOLFIN variant.
    """
    return Argument(V, 0, part)


def TrialFunction(V, part=None):
    """UFL value: Create a trial function argument to a form.

    This is the overloaded PyDOLFIN variant.

    """
    return Argument(V, 1, part)


# --- TestFunctions and TrialFunctions ---

def Arguments(V, number):
    """UFL value: Create an Argument in a mixed space, and return a
    tuple with the function components corresponding to the subelements.

    This is the overloaded PyDOLFIN variant.

    """
    if isinstance(V, MixedFunctionSpace):
        return [Argument(V.sub_space(i), number, i)
                for i in range(V.num_sub_spaces())]
    else:
        return ufl.split(Argument(V, number))


def TestFunctions(V):
    """UFL value: Create a TestFunction in a mixed space, and return a
    tuple with the function components corresponding to the
    subelements.

    This is the overloaded PyDOLFIN variant.

    """
    return Arguments(V, 0)


def TrialFunctions(V):
    """UFL value: Create a TrialFunction in a mixed space, and return a
    tuple with the function components corresponding to the
    subelements.

    This is the overloaded PyDOLFIN variant.

    """
    return Arguments(V, 1)



#!/usr/bin/python3

import numpy
from dolfin import *
from dolfin_utils.test import skip_if_not_SUNDIALS


@skip_if_not_SUNDIALS
def test_sundials_adams():

    class MyCVode(CVode):

        def derivs(self, t, u, udot):
            udot[:] = -u[:]

        def jacobian(self, v, Jv, t, y, fy):
            Jv[:] = v[:]
            return 0

        def psolve(self, t, u, udot, r, z, gamma, x, y):
            z[:] = r[:]
            return 0

    phi = Vector(MPI.comm_world, 10)
    phi[:] = 1.0

    cv = MyCVode(CVode.LMM.CV_ADAMS, CVode.ITER.CV_FUNCTIONAL)
    cv.init(phi, 1e-7, 1e-7)

    nstep = 200
    dt = 0.01
    for i in range(nstep):
        t = cv.step(dt)
        assert (exp(-t) - phi[0]) < 1e-6

@skip_if_not_SUNDIALS
def test_sundials_newton():

    class MyCVode(CVode):

        def derivs(self, t, u, udot):
            udot[:] = -u[:]

        def jacobian(self, v, Jv, t, y, fy):
            Jv[:] = v[:]
            return 0

        def psolve(self, t, u, udot, r, z, gamma, x, y):
            z[:] = r[:]
            return 0

    phi = Vector(MPI.comm_world, 10)
    phi[:] = 1.0

    cv = MyCVode(CVode.LMM.CV_BDF, CVode.ITER.CV_NEWTON)
    cv.init(phi, 1e-7, 1e-7)

    nstep = 200
    dt = 0.01
    for i in range(nstep):
        t = cv.step(dt)
        assert (exp(-t) - phi[0]) < 1e-6

@skip_if_not_SUNDIALS
def test_sundials_diffusion_1d():
    # Finite difference test
    mesh = UnitIntervalMesh(MPI.comm_world, 100)
    Q = FunctionSpace(mesh, "CG", 1)
    F = Function(Q)
    b = 2.0
    F.interpolate(Expression("exp(-b*pow(x[0] - 0.5, 2))", b=b, degree=1))
    phi = F.vector()

    class tmp_test(CVode):
        def derivs(self, t, phi, phidot):
            p = phi[:]
            pd = numpy.zeros_like(p)
            for i in range(1, len(p)-1):
                pd[i] = p[i-1] - 2*p[i] + p[i+1]
            phidot[:] = pd

        def jacobian(self, v, Jv, t, y, fy):
            Jv[:] = v[:]
            return 0
        def psolve(self, t, u, udot, r, z, gamma, x, y):
            z[:] = r[:]
            return 0

    cv = tmp_test(CVode.LMM.CV_BDF, CVode.ITER.CV_NEWTON)
    cv.init(phi, 1e-7, 1e-7)

    t0 = 1.0/(4.0*b)
    cv.set_time(t0)

    nstep = 200
    dt = 0.25
    for i in range(nstep):
        t = cv.step(dt)
        print(t, phi.max(), phi.sum(), sqrt(t)*phi.max())



"""Unit tests for the CollisionPredicates class"""

# Copyright (C) 2014 Anders Logg and August Johansson
  #
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2014-02-16
# Last changed: 2016-11-22

import pytest
from dolfin import *
from dolfin_utils.test import skip_in_parallel
import numpy as np

@skip_in_parallel
def create_mesh(a, b):
    editor = MeshEditor()
    mesh = Mesh()
    editor.open(mesh, "interval", 1, 2)
    editor.init_cells(1)
    editor.init_vertices(2)
    editor.add_cell(0, np.array( (0, 1), dtype='uint') )
    editor.add_vertex(0, np.array( (a.x(), a.y()), dtype='float'))
    editor.add_vertex(1, np.array( (b.x(), b.y()), dtype='float'))
    editor.close()
    return mesh;

@skip_in_parallel
def test_L_version_1():
    mesh0 = create_mesh(Point(0., 0.), Point(1., 0.))
    mesh1 = create_mesh(Point(0., 0.), Point(0., 1.))
    cell0 = Cell(mesh0, 0)
    cell1 = Cell(mesh1, 0)
    assert cell0.collides(cell1) == True

@skip_in_parallel
def test_L_version_2():
    # mesh0 = create_mesh(Point(np.finfo(np.float32).eps, 0.), Point(1., 0.))
    # mesh0 = create_mesh(Point(eps(), 0.), Point(1., 0.))
    mesh0 = create_mesh(Point(2.23e-15, 0.), Point(1., 0.))
    mesh1 = create_mesh(Point(0., 0.), Point(0., 1.))
    # print(mesh0.str(True))
    # print(mesh1.str(True))
    cell0 = Cell(mesh0, 0)
    cell1 = Cell(mesh1, 0)
    assert cell0.collides(cell1) == False

@skip_in_parallel
def test_L_version_3():
    # mesh0 = create_mesh(Point(np.finfo(np.float32).eps, 0.), Point(1., 0.))
    # mesh0 = create_mesh(Point(eps(), 0.), Point(1., 0.))
    a = Point(2.23e-100, 0.) # assume shewchuk works
    b = Point(1., 0.)
    c = Point(0., 0.)
    d = Point(0., 1.)

    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(a, b, c, d) == False

@skip_in_parallel
def test_aligned_version_1():
    mesh0 = create_mesh(Point(0,0), Point(1,0))
    mesh1 = create_mesh(Point(1,0), Point(2,0))
    cell0 = Cell(mesh0, 0)
    cell1 = Cell(mesh1, 0)
    assert cell0.collides(cell1) == True

@skip_in_parallel
def test_aligned_version_2():
    mesh0 = create_mesh(Point(0,0), Point(1,0))
    mesh1 = create_mesh(Point(2,0), Point(3,0))
    cell0 = Cell(mesh0, 0)
    cell1 = Cell(mesh1, 0)
    assert cell0.collides(cell1) == False

@skip_in_parallel
def test_collinear_1():
    p0 = Point(0.05, 0.15)
    p1 = Point(0.85, 0.95)
    q0 = Point(0.2875, 0.3875)
    q1 = Point(0.6125, 0.7125)
    meshp = create_mesh(p0, p1)
    meshq = create_mesh(q0, q1)
    cellp = Cell(meshp, 0)
    cellq = Cell(meshq, 0)
    assert cellp.collides(cellq) == True

@skip_in_parallel
def test_collinear_2():
    res = cpp.geometry.CollisionPredicates.collides_segment_segment_2d(Point(.5, .3),
                                                                       Point(.5, .4),
                                                                       Point(.5, .5),
                                                                       Point(.5, .6))
    assert not res

@skip_in_parallel
def test_segment_segment_2d():
    # p0 is on segment q0-q1
    p0 = Point(1e-30, 0)
    p1 = Point(1, 2)
    p2 = Point(2, 1)
    q0 = Point(1, 0)
    q1 = Point(0, 0)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p0, p1, q0, q1)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p1, p0, q0, q1)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p0, p1, q1, q0)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p1, p0, q1, q0)

    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p0, p2, q0, q1)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p2, p0, q0, q1)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p0, p2, q1, q0)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p2, p0, q1, q0)

    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p0, p1, q1, q0)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p1, p0, q1, q0)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p0, p1, q0, q1)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p1, p0, q0, q1)

    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p0, p2, q1, q0)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p2, p0, q1, q0)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p0, p2, q0, q1)
    assert cpp.geometry.CollisionPredicates.collides_segment_segment_2d(p2, p0, q0, q1)



"""This demo program solves Poisson's equation with a delta dirac point source

    - div grad u(x, y) = f(x, y)*dirac

on the unit square with source f given by

    f(x, y) = 0.4

and boundary conditions given by

    u(x, y) = 0          for x = 0 or x = 1
du/dn(x, y) = A*sin(5*x) for y = 0 or y = 1
"""

# Copyright (C) 2014 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2014-10-20
# Last changed: 2014-10-20

# Begin demo

from dolfin import *
import matplotlib.pyplot as plt

# Create mesh and define function space
mesh = UnitSquareMesh(128, 128)
V = FunctionSpace(mesh, "Lagrange", 1)

# Define Dirichlet boundary (x = 0 or x = 1)
def boundary(x):
    return x[0] < DOLFIN_EPS or x[0] > 1.0 - DOLFIN_EPS

def center_func(x):
    return (0.45 <= x[0] and x[0] <= 0.55 and near(x[1], 0.5)) or \
           0.45 <= x[1] and x[1] <= 0.55 and near(x[0], 0.5)

# Define domain for point integral
center_domain = MeshFunction("size_t", mesh, 0, 0)
center = AutoSubDomain(center_func)
center.mark(center_domain, 1)
dPP = dP(subdomain_data=center_domain)

# Define boundary condition
u0 = Constant(0.0)
bc = DirichletBC(V, u0, boundary)

# Define variational problem
u = TrialFunction(V)
v = TestFunction(V)
f = Constant(0.4)
g = Expression("A*sin(5*x[0])", A=10.0, degree=2)
a = inner(grad(u), grad(v))*dx
L = f*v*dPP(1) + g*v*ds

# Compute solution
u = Function(V)
solve(a == L, u, bc)

# Save solution in VTK format
file = File("poisson.pvd")
file << u

# Plot solution
plot(u)
plt.show()



# Copyright (C) 2015 Jan Blechta
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from dolfin import *

# Let's solve some variational problem to get non-trivial timings
mesh = UnitSquareMesh(32, 32)
V = FunctionSpace(mesh, "Lagrange", 1)
bc = DirichletBC(V, 0.0, lambda x: near(x[0], 0.0) or near(x[0], 1.0))
u, v = TrialFunction(V), TestFunction(V)
f = Expression("10*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)", degree=2)
g = Expression("sin(5*x[0])", degree=2)
a = inner(grad(u), grad(v))*dx
L = f*v*dx + g*v*ds
u = Function(V)
solve(a == L, u, bc)

# List timings; average across processes in parallel
list_timings(TimingClear.keep, [TimingType.wall, TimingType.system])

# Get Table object with timings
t = timings(TimingClear.keep,
            [TimingType.wall, TimingType.user, TimingType.system])

# Use different MPI reductions
t_sum = MPI.sum(MPI.comm_world, t)
t_min = MPI.min(MPI.comm_world, t)
t_max = MPI.max(MPI.comm_world, t)
t_avg = MPI.avg(MPI.comm_world, t)

# Print aggregate timings to screen
print('\n'+t_sum.str(True))
print('\n'+t_min.str(True))
print('\n'+t_max.str(True))
print('\n'+t_avg.str(True))

# Store to XML file on rank 0
if MPI.rank(MPI.comm_world) == 0:
    f = File(MPI.comm_self, "timings_aggregate.xml")
    f << t_sum
    f << t_min
    f << t_max
    f << t_avg

# Store timings of each rank separately
f = File(MPI.comm_self, "timings_rank_%d.xml"
         % MPI.rank(MPI.comm_world))
f << t

# Helper function for storing rank-wise average, min and max
dump_timings_to_xml("timings_avg_min_max.xml", TimingClear.clear)



# -*- coding: utf-8 -*-
"""Module for converting the  Abaqus mesh format."""

# Copyright (C) 2012 Arve Knudsen and Garth N/ Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Simon Funke (surface export)


# TODO: The change to python 3 compatible iteration may have introduced performance
#       regressions here, with unnecessary list() applications in the below code.
import re
import csv
import numpy as np

from . import xml_writer

class State:
    Init, Unknown, Invalid, ReadHeading, ReadNodes, ReadCells, \
        ReadNodeSet, ReadCellSet, ReadSurfaceSet = list(range(9))

def convert(ifilename, handler):
    """ Convert from Abaqus.

    The Abaqus format first defines a node block, then there should be a number
    of elements containing these nodes.
    """

    # Dictionary of nodes (maps node id to coordinates)
    nodes = {}

    # Dictionary of elements (maps cell id to list of cell nodes)
    elems = {}

    # Lists of nodes for given name (key)
    node_sets = {}

    # Lists of cells for given name (key)
    cell_sets = {}

    # Lists of surfaces for given name (key) in the format:
    # {'SS1': [set(['SS1_S1', 'S1']), set(['SS1_S4', 'S4'])]},
    # where SS1 is the name of the surface, SS1_S1 is the name of the
    # cell list whose first face is to be selected, ...
    surface_sets = {}

    # Open file Abaqus file
    file = open(ifilename, 'r')
    csv_file = csv.reader(file, delimiter=',', skipinitialspace=True)

    node_set_name = None
    generate = None

    # Set intial state state
    state = State.Init

    # Read data from input file
    for l in csv_file:

        # Sanity check
        if (len(l) == 0): print("Ooops, zero length.")

        if l[0].startswith('**'): # Pass over comments
            continue
        elif l[0].startswith('*'): # Have a keyword
            state = State.Unknown

            if l[0].lower() == "*heading":
                state = State.ReadHeading

            elif l[0].lower() == "*part":
                part_name = _read_part_name(l)

            elif l[0].lower() == "*end part":
                state = State.Invalid

            elif l[0].lower() == "*node":
                node_set_name = _create_node_list_entry(node_sets, l)
                state = State.ReadNodes

            elif l[0].lower() == "*element":
                cell_type, cell_set_name = _read_element_keywords(cell_sets, l)
                state = State.ReadCells

            elif l[0].lower() == "*nset":
                node_set_name, generate = _read_nset_keywords(node_sets, l)
                state = State.ReadNodeSet

            elif l[0].lower() == "*elset":
                cell_set_name, generate = _read_elset_keywords(cell_sets, l)
                if generate:
                    print("WARNING: generation of *elsets not tested.")
                state = State.ReadCellSet

            elif l[0].lower() == "*surface":
                surface_set_name, generate = _read_surface_keywords(surface_sets, l)
                state = State.ReadSurfaceSet

            else:
                print("WARNING: unrecognised Abaqus input keyword:", l[0])
                state = State.Unknown

        else:

            if state == State.ReadHeading:
                model_name = _read_heading(l)

            elif state == State.ReadNodes:
                node_id = int(l[0]) - 1
                coords = [float(c) for c in l[1:]]
                nodes[node_id] = coords
                if node_set_name is not None:
                    node_sets[node_set_name].add(node_id)

            elif state == State.ReadCells:
                cell_id = int(l[0]) - 1
                cell_connectivity = [int(v) - 1 for v in l[1:]]
                elems[cell_id] = cell_connectivity
                if cell_set_name is not None:
                    cell_sets[cell_set_name].add(cell_id)

            elif state == State.ReadNodeSet:

                try:
                    if generate:
                        n0, n1, increment = l
                        node_range = list(range(int(n0) - 1, int(n1) - 1, int(increment)))
                        node_range.append(int(n1) - 1)
                        node_sets[node_set_name].update(node_range)
                    else:
                        # Strip empty term at end of list, if present
                        if l[-1] == '': l.pop(-1)
                        node_range = [int(n) - 1 for n in l]
                        node_sets[node_set_name].update(node_range)
                except:
                    print("WARNING: Non-integer node sets not yet supported.")

            elif state == State.ReadCellSet:
                try:
                    if generate:
                        n0, n1, increment = l
                        cell_range = list(range(int(n0) - 1, int(n1) - 1, int(increment)))
                        cell_range.append(int(n1) - 1)
                        cell_sets[cell_set_name].update(cell_range)
                    else:
                        # Strip empty term at end of list, if present
                        if l[-1] == '': l.pop(-1)
                        cell_range = [int(n) - 1 for n in l]
                        cell_sets[cell_set_name].update(cell_range)
                except:
                    print("WARNING: Non-integer element sets not yet supported.")

            elif state == State.ReadSurfaceSet:
                # Strip empty term at end of list, if present
                if l[-1] == '': l.pop(-1)
                surface_sets[surface_set_name].update([tuple(l)])

            elif state == State.Invalid: # part
                raise Exception("Inavlid Abaqus parser state..")


    # Close CSV object
    file.close()
    del csv_file

    # Write data to XML file
    # Note that vertices/cells must be consecutively numbered, which
    # isn't necessarily the case in Abaqus. Therefore we enumerate and
    # translate original IDs to sequence indexes if gaps are present.

    # FIXME
    handler.set_mesh_type("tetrahedron", 3)

    process_facets = len(surface_sets) > 0
    if process_facets:
        try:
            from dolfin import MeshEditor, Mesh
        except ImportError:
            _error("DOLFIN must be installed to handle Abaqus boundary regions")

        mesh = Mesh()
        mesh_editor = MeshEditor()
        mesh_editor.open(mesh, "tetrahedron", 3, 3)

    node_ids_order = {}
    # Check for gaps in vertex numbering
    node_ids = list(nodes.keys())
    if len(node_ids) > 0:
        vertex_gap = (min(node_ids) != 0 or max(node_ids) != len(node_ids) - 1)
        for x, y in enumerate(node_ids):
            node_ids_order[y]= x  # Maps Abaqus IDs to Dolfin IDs
    else:
        vertex_gap = True

    # Check for gaps in cell numbering
    elemids = list(elems.keys())
    if len(elemids) > 0:
        cell_gap = (min(elemids) != 0 or max(elemids) != len(elemids) - 1)
    else:
        cell_gap = True

    # Write vertices to XML file
    handler.start_vertices(len(nodes))
    if process_facets:
        mesh_editor.init_vertices_global(len(nodes), len(nodes))

    if not vertex_gap:

        for v_id, v_coords in list(nodes.items()):
            handler.add_vertex(v_id, v_coords)
            if process_facets:
                mesh_editor.add_vertex(v_id, np.array(v_coords, dtype=np.float_))

    else:

        for idx, (v_id, v_coords) in enumerate(nodes.items()):
            handler.add_vertex(idx, v_coords)
            if process_facets:
                mesh_editor.add_vertex(idx, np.array(v_coords, dtype=np.float_))

    handler.end_vertices()

    # Write cells to XML file
    handler.start_cells(len(elems))
    if process_facets:
        mesh_editor.init_cells_global(len(elems), len(elems))

    if not vertex_gap and not cell_gap:

        for c_index, c_data in list(elems.items()):
            for v_id in c_data:
                if not (0 <= v_id < len(nodes)):
                    handler.error("Element %s references non-existent node %s" % (c_index, v_id))
            handler.add_cell(c_index, c_data)

            if process_facets:
                c_data_tmp = np.array(c_data)
                c_data_tmp.sort()
                mesh_editor.add_cell(c_index, np.array(c_data_tmp, dtype=np.uintp))


    elif not vertex_gap and cell_gap:

        for idx, (c_index, c_data) in enumerate(elems.items()):
            for v_id in c_data:
                if not (0 <= v_id < len(nodes)):
                    handler.error("Element %s references non-existent node %s" % (c_index, v_id))
            handler.add_cell(idx, c_data)

            if process_facets:
                c_data_tmp = np.array(c_data)
                c_data_tmp.sort()
                mesh_editor.add_cell(idx, np.array(c_data_tmp, dtype=np.uintp))

    else:

        for idx, (c_id, c_data) in enumerate(elems.items()):
            c_nodes = []
            for v_id in c_data:
                try: c_nodes.append(node_ids_order[v_id])
                except ValueError:
                    handler.error("Element %s references non-existent node %s" % (c_id, v_id))
            handler.add_cell(idx, c_nodes)

            if process_facets:
                c_nodes.sort()
                mesh_editor.add_cell(idx, np.array(c_nodes, dtype=np.uintp))

    handler.end_cells()

    # Write MeshValueCollections to XML file
    handler.start_domains()

    # Build a abaqus node ID -> dolfin cell ID map (which is not unique but that is irrelevant here)
    #                           and its local entity.
    if len(node_sets) > 0:
        node_cell_map = {}
        for c_dolfin_index, (c_index, c_data) in enumerate(elems.items()):
            c_data_tmp = np.array(c_data)
            c_data_tmp.sort()
            for local_entity, n_index in enumerate(c_data_tmp):
                node_cell_map[n_index] = (c_dolfin_index, local_entity)

    # Write vertex/node sets
    dim = 0
    for value, (name, node_set) in enumerate(node_sets.items()):
        handler.start_mesh_value_collection(name, dim, len(node_set), "uint")

        for node in node_set:
            try:
                cell, local_entity = node_cell_map[node]
                handler.add_entity_mesh_value_collection(dim, cell, value, local_entity=local_entity)
            except KeyError:
                print("Warning: Boundary references non-existent node %s" % node)
        handler.end_mesh_value_collection()

    # Write cell/element sets
    dim = 3
    for name, s in list(cell_sets.items()):
        handler.start_mesh_value_collection(name, dim, len(s), "uint")
        for cell in s:
            handler.add_entity_mesh_value_collection(dim, cell, 0)
        handler.end_mesh_value_collection()

    # Write surface sets
    if process_facets:
        dim = 2
        nodes_facet_map = _nodes_facet_map(mesh)

        data = [int(0)] * mesh.num_facets()
        S1 = [0, 1, 2]
        S2 = [0, 3, 1]
        S3 = [1, 3, 2]
        S4 = [2, 3, 0]
        node_selector = {'S1': S1,
                         'S2': S2,
                         'S3': S3,
                         'S4': S4,
                         }

        for index, (name, s) in enumerate(surface_sets.items()):
            cell_face_list = []
            for cell_set_name, face_index in s:
                cell_face_list += [(cell, face_index) for cell in cell_sets[cell_set_name]]

            for cell, face in cell_face_list:
                cell_nodes = elems[cell]
                # Extract the face nodes
                face_nodes = [cell_nodes[i] for i in node_selector[face]]
                dolfin_face_nodes = [node_ids_order[n] for n in face_nodes]
                dolfin_face_nodes.sort()
                # Convert the face_nodes to dolfin IDs
                face_id = nodes_facet_map[tuple(dolfin_face_nodes)]
                data[face_id] = index + 1

        # Create and initialise the mesh function
        handler.start_meshfunction("facet_region", dim, mesh.num_facets() )
        for index, physical_region in enumerate (data):
            handler.add_entity_meshfunction(index, physical_region)
        handler.end_meshfunction()


    handler.end_domains()

def _nodes_facet_map(mesh):
    # Now process the facet markers
    dim = 2
    mesh.init(dim, 0)
    facets_as_nodes = mesh.topology()(dim, 0)().reshape(mesh.num_facets(), 3)

    # Build the reverse map
    nodes_as_facets = {}
    for facet in range(mesh.num_facets()):
        nodes_as_facets[tuple(facets_as_nodes[facet,:])] = facet

    return nodes_as_facets


def _read_heading(l):
    return l[0].strip()


def _read_part_name(l):

    if (len(l) < 2): print("Ooops, length problem.")
    part_names = l[1].split('=')

    if (len(part_names) < 2): print("Ooops, part names length problem.")
    return part_names[1].strip()


def _create_node_list_entry(node_sets, l):

    # Check for node set name
    node_set_name = None
    if len(l) == 2:
        set_data = l[1].split('=')
        assert len(set_data) == 2, "wrong list length"
        if set_data[0].lower() == "nset":
            node_set_name = set_data[1]
            if node_set_name not in node_sets:
                node_sets[node_set_name] = set()
    return node_set_name

def _read_element_keywords(cell_sets, l):

    # Get element type and element set name
    element_type = None
    element_set_name = None
    for key in l[1:]:
        key_parts = key.split('=')
        key_name = key_parts[0].lower().strip()
        if key_name == "type":
            element_type = key_parts[1].lower().strip()
        elif key_name == "elset":
            element_set_name = key_parts[1].strip()

    # Test that element is supported
    check_element_support(element_type)

    # Add empty set to cell_sets dictionary
    if element_set_name:
        if element_set_name not in cell_sets:
            cell_sets[element_set_name] = set()

    return element_type, element_set_name


def _read_nset_keywords(node_sets, l):

    node_set_name = None
    generate = None

    # Get set name and add to dict
    set_data = l[1].split('=')
    assert len(set_data) == 2, "wrong list length, set name missing"
    assert set_data[0].lower() == "nset"
    node_set_name = set_data[1]
    if node_set_name not in node_sets:
        node_sets[node_set_name] = set()

    # Check for generate flag
    if len(l) == 3:
        if l[2].lower() == "generate":
           generate = True

    return node_set_name, generate


def _read_elset_keywords(sets, l):

    set_name = None
    generate = None

    # Get set name and add to dict
    set_data = l[1].split('=')
    assert len(set_data) == 2, "wrong list length, set name missing"
    assert set_data[0].lower() == "elset"
    set_name = set_data[1]
    if set_name not in sets: sets[set_name] = set()

    # Check for generate flag
    if len(l) == 3:
        if l[2].lower() == "generate":
            generate = True

    return set_name, generate

def _read_surface_keywords(sets, l):

    surface_name = None
    generate = None

    # Get surface name and add to dict
    surface_data = l[1].split('=')
    assert len(surface_data) == 2, "wrong list length, surface name missing"
    assert surface_data[0].lower() == "name"
    surface_name = surface_data[1]
    if surface_name not in sets: sets[surface_name] = set()

    generate = False
    return surface_name, generate


def check_element_support(element_type):
    supported_elements = ('c3d4',)
    if element_type.lower() not in supported_elements:
        raise Exception("Element type not supported.")



# Copyright (C) 2009 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2009-11-11
# Last changed: 2012-11-12

from dolfin import *
from numpy import ones
import matplotlib.pyplot as plt


# TimeSeries requires DOLFIN to be configured with HDF5
if has_hdf5() is False:
    print("This demo requires DOLFIN to be configured with HDF5")
    exit()

# Create empty time series
series = TimeSeries(MPI.comm_world, "primal")

# Create a mesh and a vector
mesh = UnitSquareMesh(2, 2)
x = Vector()

# Add a bunch of meshes and vectors to the series
t = 0.0
while t < 1.0:

    # Refine mesh
    mesh = refine(mesh);

    # Set some vector values
    x = Vector(mesh.mpi_comm(), mesh.num_vertices());
    x[:] = ones(x.size())

    # Append to series
    series.store(mesh, t)
    series.store(x, t)

    t += 0.2

# Retrieve mesh and vector at some point in time
series.retrieve(mesh, 0.29)
x = Vector()
series.retrieve(x, 0.31, False)

# Plot mesh
plot(mesh)
plt.show()



"""Unit tests for the LUSolver interface"""

# Copyright (C) 2014 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from dolfin import *
import pytest
from dolfin_utils.test import skip_if_not_PETSc, skip_in_parallel

backends = ["PETSc", pytest.param(("Eigen"), marks=skip_in_parallel)]

@pytest.mark.parametrize('backend', backends)
def test_lu_solver(backend):

    # Check whether backend is available
    if not has_linear_algebra_backend(backend):
        pytest.skip('Need %s as backend to run this test' % backend)

    # Set linear algebra backend
    prev_backend = parameters["linear_algebra_backend"]
    parameters["linear_algebra_backend"] = backend

    mesh = UnitSquareMesh(12, 12)
    V = FunctionSpace(mesh, "Lagrange", 1)
    u, v = TrialFunction(V), TestFunction(V)
    A = assemble(Constant(1.0)*u*v*dx)
    b = assemble(Constant(1.0)*v*dx)

    norm = 13.0

    solver = LUSolver()
    x = Vector()
    solver.solve(A, x, b)
    assert round(x.norm("l2") - norm, 10) == 0

    solver = LUSolver(A)
    x = Vector()
    solver.solve(x, b)
    assert round(x.norm("l2") - norm, 10) == 0

    solver = LUSolver()
    x = Vector()
    solver.set_operator(A)
    solver.solve(x, b)
    assert round(x.norm("l2") - norm, 10) == 0

    solver = LUSolver()
    x = Vector()
    solver.solve(A, x, b)
    assert round(x.norm("l2") - norm, 10) == 0

    # Reset backend
    parameters["linear_algebra_backend"] = prev_backend


@pytest.mark.parametrize('backend', backends)
def test_lu_solver_reuse(backend):
    """Test that LU re-factorisation is only performed after
    set_operator(A) is called"""

    # Test requires PETSc version 3.5 or later. Use petsc4py to check
    # version number.
    try:
        from petsc4py import PETSc
    except ImportError:
        pytest.skip("petsc4py required to check PETSc version")
    else:
        if not PETSc.Sys.getVersion() >= (3, 5, 0):
            pytest.skip("PETSc version must be 3.5  of higher")


    # Check whether backend is available
    if not has_linear_algebra_backend(backend):
        pytest.skip('Need %s as backend to run this test' % backend)

    # Set linear algebra backend
    prev_backend = parameters["linear_algebra_backend"]
    parameters["linear_algebra_backend"] = backend

    mesh = UnitSquareMesh(12, 12)
    V = FunctionSpace(mesh, "Lagrange", 1)
    u, v = TrialFunction(V), TestFunction(V)
    b = assemble(Constant(1.0)*v*dx)

    A = assemble(Constant(1.0)*u*v*dx)
    norm = 13.0

    solver = LUSolver(A)
    x = Vector()
    solver.solve(x, b)
    assert round(x.norm("l2") - norm, 10) == 0

    assemble(Constant(0.5)*u*v*dx, tensor=A)
    x = Vector()
    solver.solve(x, b)
    if backend != 'Eigen':
        # The eigen backend only recomputes the factorization once set_operator is called
        assert round(x.norm("l2") - 2.0*norm, 10) == 0

    solver.set_operator(A)
    solver.solve(x, b)
    assert round(x.norm("l2") - 2.0*norm, 10) == 0

    # Reset backend
    parameters["linear_algebra_backend"] = prev_backend



# -*- coding: utf-8 -*-
# Copyright (C) 2017 Jan Blechta
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import functools
from dolfin import cpp

__all__ = ["Timer", "timed"]


class Timer(cpp.common.Timer):
    """A timer can be used for timing tasks. The basic usage is::

        with Timer("Some costly operation"):
            costly_call_1()
            costly_call_2()

    or::

        with Timer() as t:
            costly_call_1()
            costly_call_2()
            print("Ellapsed time so far: %s" % t.elapsed()[0])

    The timer is started when entering context manager and timing
    ends when exiting it. It is also possible to start and stop a
    timer explicitly by::

        t.start()
        t.stop()

    and retrieve timing data using::

        t.elapsed()

    Timings are stored globally (if task name is given) and
    may be printed using functions ``timing``, ``timings``,
    ``list_timings``, ``dump_timings_to_xml``, e.g.::

        list_timings(TimingClear.keep, [TimingType.wall, TimingType.user])
    """

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, *args):
        self.stop()


def timed(task):
    """Decorator for timing functions. Usage::

        @timed("Do Foo")
        def do_foo(*args, **kwargs):
            # Do something costly
            pass

        do_foo()

        list_timings(TimingClear.keep, [TimingType.wall, TimingType.user])

        t = timing("Do Foo", TimingClear.clear)
        print("Do foo wall time: %s" % t[1])
    """
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            with Timer(task):
                return func(*args, **kwargs)
        return wrapper
    return decorator



# Copyright (C) 2010 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2010-02-08
# Last changed: 2010-02-09

from dolfin import *
import matplotlib.pyplot as plt


# Create mesh and function spaces
mesh = UnitSquareMesh(8, 8)
P1 = FunctionSpace(mesh, "CG", 1)
P2 = FunctionSpace(mesh, "CG", 2)

# Create exact dual
dual = Expression("sin(5.0*x[0])*sin(5.0*x[1])", degree=2)

# Create P1 approximation of exact dual
z1 = Function(P1)
z1.interpolate(dual)

# Create P2 approximation from P1 approximation
z2 = Function(P2)
z2.extrapolate(z1)

# Plot approximations
plt.figure()
plot(z1, title="z1")
plt.figure()
plot(z2, title="z2")
plt.show()



import dolfin.function
import dolfin.cpp as cpp


# Functions to extend cpp.io.File with

def __lshift__(self, u):
    """Support for the legacy '<<' output to file syntax"""

    # Note: __lshift__ notation for IO will be deprecated, see
    # https://bitbucket.org/fenics-project/dolfin/issues/895.

    if isinstance(u, dolfin.function.function.Function):
        self.write(u._cpp_object)
    elif isinstance(u, tuple):
        if isinstance(u[0], dolfin.function.function.Function):
            self.write(u[0]._cpp_object, u[1])
        else:
            self.write(u[0], u[1])
    else:
        self.write(u)


def __rshift__(self, u):
    """Support for the legacy '>>' read from file syntax"""

    # Note: __rshift__ notation for IO will be deprecated, see
    # https://bitbucket.org/fenics-project/dolfin/issues/895.

    if isinstance(u, dolfin.function.function.Function):
        self.read(u._cpp_object)
    else:
        self.read(u)


# Extend cpp.io.File class, and clean-up
cpp.io.File.__lshift__ = __lshift__
cpp.io.File.__rshift__ = __rshift__
del __lshift__, __rshift__



"""This demo demonstrates the DOLFIN parameter system.

Try running this demo with

  python demo.py --bar 1 --solver_parameters.max_iterations 1000
"""

# Copyright (C) 2009 Johan Hake and Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2009-09-06
# Last changed: 2009-12-08


from dolfin import *

#--- Demo of global DOLFIN parameters ---

MPI.init()

# Set some global DOLFIN parameters
parameters["linear_algebra_backend"] = "Eigen"

# Print global DOLFIN parameters
info(parameters, True)
print()

# Read parameters from file
file = File("parameters.xml")
parameters = Parameters("parameters")
file >> parameters
info(parameters, True)
print()

#--- Demo of nested parameter sets ---

# Create an application parameter set
application_parameters = Parameters("application_parameters")

# Create application parameters
application_parameters.add("foo", 1.0)
application_parameters.add("bar", 100)
application_parameters.add("baz", False)
application_parameters.add("pc", "amg")

# Create a solver parameter set
solver_parameters = Parameters("solver_parameters")

# Create solver parameters
solver_parameters.add("max_iterations", 100)
solver_parameters.add("tolerance", 1e-16)
solver_parameters.add("relative_tolerance", 1e-16, 1e-16, 1.0)

# Set range for parameter
solver_parameters.set_range("max_iterations", 0, 1000)

# Set some parameter values
solver_parameters["max_iterations"] = 500
solver_parameters["relative_tolerance"] = 0.1

# Set solver parameters as nested parameters of application parameters
application_parameters.add(solver_parameters)

# Parse command-line options
application_parameters.parse()

# Access parameter values
foo = application_parameters["foo"]
bar = application_parameters["bar"]
tol = application_parameters["solver_parameters"]["tolerance"]

# Print parameter values
print("foo =", foo)
print("bar =", bar)
print("tol =", tol)
print()

# Print application parameters
info(application_parameters, True)
print()

#--- Demo of Krylov solver parameters ---

# Set a parameter for the Krylov solver
solver = KrylovSolver()
solver.parameters["relative_tolerance"] = 1e-20

# Print Krylov solver parameters
info(solver.parameters, True)
print()

#--- Demo of updating a parameter set ---

# Create a subset of the application parameter set
parameter_subset = Parameters("parameter_subset")
parameter_subset.add("foo", 3.0)
nested_subset = Parameters("solver_parameters")
nested_subset.add("max_iterations", 850)
parameter_subset.add(nested_subset)

# Update application parameters
application_parameters.update(parameter_subset)
info(application_parameters, True)

# Can also update using a dictionary
parameter_subset = dict(foo = 1.5, solver_parameters = dict(max_iterations = 50))
application_parameters.update(parameter_subset)

# Or like this
parameter_subset = {"foo": 1.5, "solver_parameters": {"max_iterations": 50}}
application_parameters.update(parameter_subset)

print("Features below not implemented in pybind11")
exit()

# Print command-line option string
print("\nCommand-line option string")
print(application_parameters.option_string())

# Demostrate access to full info of parameters
def print_data(par, indent=""):
    print("\n" + indent + "Content of:", par.name())
    for key, par_info in par.iterdata():
        if isinstance(par_info,Parameters):
            print_data(par_info, indent + "    ")
        else:
            print(indent + key, par_info)

print_data(application_parameters)

# Direct creation of application parameter set
new_application_parameters = Parameters(
    "application_parameters",
    foo = 1.0,
    bar = 100,
    pc = "amg",
    solver_parameters = Parameters(
        "solver",
        max_iterations = 100,
        tolerance = 1e-16,
        relative_tolerance = (1e-16, 1e-16, 1.0),
        pcs = ("ilu", ["ilu","amg","icc","sor"])
    )
)



# -*- coding: utf-8 -*-
"""This module provides functionality for form assembly in Python,
corresponding to the C++ assembly and PDE classes.

The C++ :py:class:`assemble <dolfin.cpp.assemble>` function
(renamed to cpp_assemble) is wrapped with an additional
preprocessing step where code is generated using the
FFC JIT compiler.

The C++ PDE classes are reimplemented in Python since the C++ classes
rely on the dolfin::Form class which is not used on the Python side.

"""

# Copyright (C) 2007-2015 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import ufl
import dolfin.cpp as cpp
from dolfin.fem.form import Form
from dolfin import MPI
from dolfin.function.multimeshfunction import MultiMeshFunction

from ufl.form import sub_forms_by_domain

__all__ = ["assemble", "assemble_mixed", "assemble_local", "assemble_system",
           "assemble_multimesh", "SystemAssembler"]


def _create_dolfin_form(form, form_compiler_parameters=None,
                        function_spaces=None):
    if form is None:
        return form
    # First check if we got a cpp.Form
    elif isinstance(form, cpp.fem.Form):

        # Check that jit compilation has already happened
        if not hasattr(form, "_compiled_form"):
            raise TypeError("Expected a dolfin form to have a _compiled_form attribute.")

        # Warn that we don't use the parameters if we get any
        if form_compiler_parameters is not None:
            cpp.warning("Ignoring form_compiler_parameters when passed a dolfin Form!")
        return form
    elif isinstance(form, ufl.Form):
        return Form(form,
                    form_compiler_parameters=form_compiler_parameters,
                    function_spaces=function_spaces)
    else:
        raise TypeError("Invalid form type %s" % (type(form),))


def assemble_local(form, cell, form_compiler_parameters=None):
    """JIT assemble_local"""
    # Create dolfin Form object
    if isinstance(form, cpp.fem.Form):
        dolfin_form = form
    else:
        dolfin_form = _create_dolfin_form(form, form_compiler_parameters)
    result = cpp.fem.assemble_local(dolfin_form, cell)
    if result.shape[1] == 1:
        if result.shape[0] == 1:
            result = result[0][0]
        else:
            result = result.reshape((result.shape[0]))
    return result


def assemble(form, tensor=None, form_compiler_parameters=None,
             add_values=False, finalize_tensor=True,
             keep_diagonal=False, backend=None):
    """Assemble the given form and return the corresponding tensor.

    *Arguments*
        Depending on the input form, which may be a functional, linear
        form, bilinear form or higher rank form, a scalar value, a vector,
        a matrix or a higher rank tensor is returned.

    In the simplest case, no additional arguments are needed. However,
    additional arguments may and must in some cases be provided as
    outlined below.

    The ``form`` can be either a UFL Form or a DOLFIN Form.

    If the form defines integrals over different subdomains,
    :py:class:`MeshFunctions <dolfin.cpp.MeshFunction>` over the
    corresponding topological entities defining the subdomains can be
    provided.

    The implementation of the returned tensor is determined by the
    default linear algebra backend. This can be overridden by
    specifying a different backend.

    Each call to assemble() will create a new tensor. If the
    ``tensor`` argument is provided, this will be used instead.
    Sparsity pattern of ``tensor`` is reset iff ``tensor.empty()``
    holds.

    If the ``keep_diagonal`` is set to True, assembler ensures that
    potential zeros on a matrix diagonal are kept in sparsity pattern
    so every diagonal entry can be changed in a future (for example by
    ident() or ident_zeros()).

    Specific form compiler parameters can be provided by the
    ``form_compiler_parameters`` argument. Form compiler parameters
    can also be controlled using the global parameters stored in
    parameters["form_compiler"].

    *Examples of usage*
        The standard stiffness matrix ``A`` and a load vector ``b``
        can be assembled as follows:

        .. code-block:: python

            A = assemble(inner(grad(u),grad(v))*dx)
            b = assemble(f*v*dx)

        To prescribe the domains over which integrals will be
        evaluated, create a Measure with the MeshFunction passed as
        subdomain_data.  For instance, using a mesh function marking
        parts of the boundary:

        .. code-block:: python

            # MeshFunction marking boundary parts
            boundary_markers = MeshFunction("size_t", mesh, mesh.topology().dim()-1)
            # ... fill values in boundary_markers

            # Measures with references to cell and boundary markers
            ds = Measure("ds", subdomain_data=boundary_markers)

            # Sample variational forms
            a = inner(grad(u), grad(v))*dx + p*u*v*ds(0)
            L = f*v*dx - g*v*ds(1) + p*q*v*ds(0)

            A = assemble(a)
            b = assemble(L)

        For interior facet integrals (dS), cell markers can be used to
        specify which cell is '+' and which is '-'. The '+' and '-'
        sides are chosen such that the cell marker value in the cell
        at the '+' side cell is larger than the cell marker value in
        the cell at the '-' side cell. If the values are equal or the
        cell markers are not provided, the sides are chosen
        arbitrarily.

        Note that currently, cell markers must be associated with a
        cell type integral (dx), and if you don't have such an
        integral a workaround is to add the integral of something over
        an empty domain such as 'f*dx(99)' with 99 a number not
        occuring among the cell markers. A better interface to handle
        this case will be provided later.

        .. code-block:: python

            # MeshFunctions marking boundary and cell parts
            boundary_markers = MeshFunction("size_t", mesh, mesh.topology().dim()-1)
            cell_markers = MeshFunction("size_t", mesh, mesh.topology().dim())
            # ... fill values in boundary_markers

            # Measures with references to cell and boundary markers
            ds = Measure("ds", domain=mesh, subdomain_data=boundary_markers)
            dx = Measure("dx", domain=mesh, subdomain_data=cell_markers)

            # Sample variational forms
            a = inner(grad(u), grad(v))*dx + p*u*v*ds(0)
            L = v*dx(99) - g*v*ds(1) + p*q*v*ds(0)

            A = assemble(a)
            b = assemble(L)

        To ensure that the assembled matrix has the right type, one may use
        the ``tensor`` argument:

        .. code-block:: python

            A = PETScMatrix()
            assemble(a, tensor=A)

        The form ``a`` is now assembled into the PETScMatrix ``A``.

    """

    # Create dolfin Form object
    if isinstance(form, cpp.fem.Form):
        dolfin_form = form
    else:
        dolfin_form = _create_dolfin_form(form, form_compiler_parameters)

    # Create tensor
    comm = dolfin_form.mesh().mpi_comm()
    tensor = _create_tensor(comm, form, dolfin_form.rank(), backend, tensor)

    # Create C++ assembler
    assembler = cpp.fem.Assembler()

    # Set assembler options
    assembler.add_values = add_values
    assembler.finalize_tensor = finalize_tensor
    assembler.keep_diagonal = keep_diagonal

    # Call C++ assemble
    assembler.assemble(tensor, dolfin_form)

    # Convert to float for scalars
    if dolfin_form.rank() == 0:
        tensor = tensor.get_scalar_value()

    # Return value
    return tensor

# JIT assembler


def assemble_mixed(form,
                   tensor=None,
                   form_compiler_parameters=None,
                   add_values=False,
                   finalize_tensor=True,
                   keep_diagonal=False,
                   backend=None):

    # Create dolfin Form object referencing all data needed by assembler
    if isinstance(form, cpp.fem.Form):
        dolfin_forms = [form]
    else:
        dolfin_forms = []
        for subform in sub_forms_by_domain(form):
            dolfin_forms.append(_create_dolfin_form(subform, form_compiler_parameters))

    # Create tensor
    comm = dolfin_forms[0].mesh().mpi_comm()
    tensor = _create_tensor(comm, form, dolfin_forms[0].rank(), backend, tensor)

    # Create C++ mixed assembler
    assembler = cpp.fem.MixedAssembler()

    # Set assembler options
    assembler.add_values = add_values
    assembler.finalize_tensor = finalize_tensor
    assembler.keep_diagonal = keep_diagonal

    # Call C++ assemble
    for k, dolfin_form in enumerate(dolfin_forms):
        assembler.add_values = bool(k > 0)
        assembler.assemble(tensor, dolfin_form)

    # Convert to float for scalars
    if dolfin_forms[0].rank() == 0:
        tensor = tensor.get_scalar_value()

    # Return value
    return tensor

# JIT multimesh assembler


def assemble_multimesh(form,
                       tensor=None,
                       form_compiler_parameters=None,
                       backend=None):
    "Assemble the given multimesh form and return the corresponding tensor."

    # The form that comes in is (by construction in function.Argument)
    # defined on the first part of the multimesh. We now need to create
    # the DOLFIN Forms with the proper function spaces for each part.

    # FIXME: This code makes a number of assumptions and will need to
    # be revisited and improved.

    # Warn that we don't use the parameters if we get any
    if form_compiler_parameters is not None:
        cpp.warning("Ignoring form_compiler_parameters when passed a dolfin Form!")
    form_compiler_parameters = None

    # Extract arguments and multimesh function space
    coefficients = form.coefficients()
    arguments = form.arguments()

    # Extract rank
    rank = len(arguments)

    # Extract multimesh function spaces for arguments
    V_multi = [v._V_multi for v in arguments]

    # Extract number of parts, the multimesh and create the multimesh form
    num_parts = None
    if rank > 0:
        num_parts = V_multi[0].num_parts()
        multimesh_form = cpp.fem.MultiMeshForm(*V_multi)
        multimesh = V_multi[0].multimesh()
    elif len(coefficients) > 0:
        for coeff in coefficients:
            # Only create these variables once
            if isinstance(coeff, MultiMeshFunction):
                multimesh = coeff.function_space().multimesh()
                num_parts = coeff.function_space().num_parts()
                multimesh_form = cpp.fem.MultiMeshForm(multimesh)
                break

    if not num_parts:
        # Handle the case Constant(1)*dx(domain=multimesh)
        multimesh = form.ufl_domains()[0].ufl_cargo()
        num_parts = multimesh.num_parts()
        multimesh_form = cpp.fem.MultiMeshForm(multimesh)

    # Build multimesh DOLFIN form
    for part in range(num_parts):
        # Extract standard function spaces for all arguments on
        # current part
        function_spaces = [V_multi[i].part(part) for i in range(rank)]
        # Wrap standard form
        dolfin_form = _create_dolfin_form(form,
                                          form_compiler_parameters,
                                          function_spaces)

        # Setting coefficients for the multimesh form
        for i in range(len(coefficients)):
            if isinstance(coefficients[i], MultiMeshFunction):
                coeff = coefficients[i].part(part)
            else:
                coeff = coefficients[i]
            # Developer note: This may be done more elegantly by modifiying
            # _create_dolfin_form
            dolfin_form.set_coefficient(i, coeff._cpp_object)
            dolfin_form.coefficients[i] = coeff

        # Add standard mesh to the standard form and the
        # standard form to the multimesh form
        dolfin_form.set_mesh(multimesh.part(part))
        multimesh_form.add(dolfin_form)

    for i, coeff in enumerate(coefficients):
        if isinstance(coeff, MultiMeshFunction):
            multimesh_form.set_multimesh_coefficient(i, coeff._cpp_object)

    # Build multimesh form
    multimesh_form.build()

    # Create tensor
    comm = MPI.comm_world
    tensor = _create_tensor(comm, form, rank, backend, tensor)

    # Call C++ assemble function
    assembler = cpp.fem.MultiMeshAssembler()
    assembler.assemble(tensor, multimesh_form)

    # Convert to float for scalars
    if rank == 0:
        tensor = tensor.get_scalar_value()

    # Return value
    return tensor


def assemble_system(A_form, b_form, bcs=None, x0=None,
                    form_compiler_parameters=None, add_values=False,
                    finalize_tensor=True, keep_diagonal=False,
                    A_tensor=None, b_tensor=None, backend=None):
    """Assemble form(s) and apply any given boundary conditions in a
    symmetric fashion and return tensor(s).

    The standard application of boundary conditions does not
    necessarily preserve the symmetry of the assembled matrix. In
    order to perserve symmetry in a system of equations with boundary
    conditions, one may use the alternative assemble_system instead of
    multiple calls to :py:func:`assemble
    <dolfin.fem.assembling.assemble>`.

    *Examples of usage*

       For instance, the statements

       .. code-block:: python

           A = assemble(a)
           b = assemble(L)
           bc.apply(A, b)

       can alternatively be carried out by

       .. code-block:: python

           A, b = assemble_system(a, L, bc)

       The statement above is valid even if ``bc`` is a list of
       :py:class:`DirichletBC <dolfin.fem.bcs.DirichletBC>`
       instances. For more info and options, see :py:func:`assemble
       <dolfin.fem.assembling.assemble>`.

    """
    # Create dolfin Form objects referencing all data needed by
    # assembler
    A_dolfin_form = _create_dolfin_form(A_form, form_compiler_parameters)
    b_dolfin_form = _create_dolfin_form(b_form, form_compiler_parameters)

    # Create tensors
    comm_A = A_dolfin_form.mesh().mpi_comm()
    comm_b = A_dolfin_form.mesh().mpi_comm()
    A_tensor = _create_tensor(comm_A, A_form, A_dolfin_form.rank(), backend,
                              A_tensor)
    b_tensor = _create_tensor(comm_b, b_form, b_dolfin_form.rank(), backend,
                              b_tensor)

    # Check bcs
    bcs = _wrap_in_list(bcs, 'bcs', cpp.fem.DirichletBC)

    # Call C++ assemble function
    assembler = cpp.fem.SystemAssembler(A_dolfin_form, b_dolfin_form, bcs)
    assembler.add_values = add_values
    assembler.finalize_tensor = finalize_tensor
    assembler.keep_diagonal = keep_diagonal
    if x0 is not None:
        assembler.assemble(A_tensor, b_tensor, x0)
    else:
        assembler.assemble(A_tensor, b_tensor)

    return A_tensor, b_tensor


def _wrap_in_list(obj, name, types=type):
    if obj is None:
        lst = []
    elif hasattr(obj, '__iter__'):
        lst = list(obj)
    else:
        lst = [obj]
    for obj in lst:
        if not isinstance(obj, types):
            raise TypeError("expected a (list of) %s as '%s' argument" %
                            (str(types), name))
    return lst


def _create_tensor(mpi_comm, form, rank, backend, tensor):
    """Create tensor for form"""

    # Check if tensor is supplied by user
    if tensor is not None:
        return tensor

    # Check backend argument
    if (backend is not None) and (not isinstance(backend, cpp.la.GenericLinearAlgebraFactory)):
        raise TypeError("Provide a GenericLinearAlgebraFactory as 'backend'")

    # Create tensor
    if rank == 0:
        tensor = cpp.la.Scalar(mpi_comm)
    elif rank == 1:
        if backend:
            tensor = backend.create_vector(mpi_comm)
        else:
            tensor = cpp.la.Vector(mpi_comm)
    elif rank == 2:
        if backend:
            tensor = backend.create_matrix(mpi_comm)
        else:
            tensor = cpp.la.Matrix(mpi_comm)
    else:
        raise RuntimeError("Unable to create tensors of rank %d." % rank)

    return tensor


class SystemAssembler(cpp.fem.SystemAssembler):
    __doc__ = cpp.fem.SystemAssembler.__doc__

    def __init__(self, A_form, b_form, bcs=None,
                 form_compiler_parameters=None):
        """
        Create a SystemAssembler

        * Arguments *
           a (ufl.Form, _Form_)
              Bilinear form
           L (ufl.Form, _Form_)
              Linear form
           bcs (_DirichletBC_)
              A list or a single DirichletBC (optional)
        """

        if isinstance(A_form, list) and isinstance(b_form, list):
            A_dolfin_forms = [_create_dolfin_form(f, form_compiler_parameters) for f in A_form]
            b_dolfin_forms = [_create_dolfin_form(f, form_compiler_parameters) for f in b_form]

            # Call C++ assemble function
            cpp.fem.SystemAssembler.__init__(self, A_dolfin_forms, b_dolfin_forms,
                                             bcs)

            # Keep Python counterpart of bcs (and Python object it owns)
            # alive
            self._bcs = bcs
        else:
            # Create dolfin Form objects referencing all data needed by
            # assembler
            A_dolfin_form = _create_dolfin_form(A_form, form_compiler_parameters)
            b_dolfin_form = _create_dolfin_form(b_form, form_compiler_parameters)

            # Check bcs
            bcs = _wrap_in_list(bcs, 'bcs', cpp.fem.DirichletBC)

            # Call C++ assemble function
            cpp.fem.SystemAssembler.__init__(self, A_dolfin_form, b_dolfin_form,
                                             bcs)

            # Keep Python counterpart of bcs (and Python object it owns)
            # alive
            self._bcs = bcs



import os


def get_pybind_include():
    """Find the pybind11 include path"""

    # Look in PYBIND11_DIR
    pybind_dir = os.getenv('PYBIND11_DIR', None)
    if pybind_dir:
        p = os.path.join(pybind_dir, "include")
        if (_check_pybind_path(p)):
            return [p]

    # Extract from pybind11 module
    import pybind11
    return [pybind11.get_include(True), pybind11.get_include()]


def _check_pybind_path(root):
    p = os.path.join(root, "pybind11", "pybind11.h")
    if os.path.isfile(p):
        return True
    else:
        return False



#!/usr/bin/env py.test

"""Unit tests for MeshViewMapping class"""

# Copyright (C) 2011 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2011-02-26
# Last changed: 2014-05-30

import pytest
from dolfin import *
from dolfin_utils.test import fixture, skip_in_parallel


@fixture
def cube():
    return UnitCubeMesh(5, 5, 5)


@fixture
def square():
    return UnitSquareMesh(5, 5)


@pytest.fixture(scope='module', params=range(2))
def meshes(cube, square, request):
    mesh = [cube, square]
    return mesh[request.param]

def test_make_facets_view(square, cube):
    """Create view of facets"""

    def _check_facets_view(mesh):
        marker = FacetFunction("size_t", mesh, 0)
        D = mesh.topology().dim()
        mesh.init(D-1, D)

        c = 0
        for f in facets(mesh):
            if f.num_global_entities(D) == 1:
                marker[f] = 1
                c += 1

        m2 = MeshViewMapping.create_from_marker(marker, 1)

        assert m2.num_cells() == c
        assert m2.num_cells() == len(m2.topology().mapping.cell_map())
        assert m2.num_vertices() == len(m2.topology().mapping.vertex_map())

    _check_facets_view(square)
    _check_facets_view(cube)


def test_make_cells_view(square, cube):
    """Create view of cells"""

    def _check_cells_view(mesh):
        marker = CellFunction("size_t", mesh, 0)

        ct = 0
        for c in cells(mesh):
            if c.midpoint().x() < 0.5:
                marker[c] = 1
                ct += 1

        m2 = MeshViewMapping.create_from_marker(marker, 1)

        assert m2.num_cells() == ct
        assert m2.num_cells() == len(m2.topology().mapping.cell_map())
        assert m2.num_vertices() == len(m2.topology().mapping.vertex_map())

    _check_cells_view(square)
    _check_cells_view(cube)


def test_make_edges_view(cube):
    """Create view of edges"""

    marker = EdgeFunction("size_t", cube, 0)
    cube.init(1, 3)

    c = 0
    for e in edges(cube):
        if (e.midpoint().x() == 0 or e.midpoint().x() == 1):
            marker[e] = 1
            c += 1
        elif (e.midpoint().y() == 0 or e.midpoint().y() == 1):
            marker[e] = 1
            c += 1
        elif (e.midpoint().z() == 0 or e.midpoint().z() == 1):
            marker[e] = 1
            c += 1

    m2 = MeshViewMapping.create_from_marker(marker, 1)

    assert m2.num_cells() == c
    assert m2.num_cells() == len(m2.topology().mapping.cell_map())
    assert m2.num_vertices() == len(m2.topology().mapping.vertex_map())



"""Unit tests for the Scalar interface"""

# Copyright (C) 2011 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from dolfin import *


def test_scalar_parallel_sum():
    a = Scalar()
    b = 1.0
    a.add_local_value(b)
    a.apply("add")
    assert round(a.get_scalar_value() - b*MPI.size(a.mpi_comm()), 7) == 0


def test_scalar_assembly():
    mesh = UnitSquareMesh(3, 3)

    S = Scalar()
    assemble(Constant(1.0)*dx(mesh), tensor=S)
    assemble(Constant(1.0)*dx(mesh), tensor=S)
    assert round(S.get_scalar_value() - 1.0, 7) == 0

    S = Scalar()
    assemble(Constant(1.0)*dx(mesh), tensor=S)
    assemble(Constant(1.0)*dx(mesh), tensor=S, add_values=True)
    assert round(S.get_scalar_value() - 2.0, 7) == 0

    S = Scalar()
    assemble(Constant(1.0)*dx(mesh), tensor=S, add_values=True)
    assemble(Constant(1.0)*dx(mesh), tensor=S, add_values=True)
    assert round(S.get_scalar_value() - 2.0, 7) == 0



"""Unit tests for coordinate derivative"""

# Copyright (C) 2018 Florian Wechsung
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Florian Wechsung 2018 and Jørgen S. Dokken 2020

import pytest
import numpy as np
from dolfin import *
from ufl import replace
from ufl.log import UFLException
from dolfin_utils.test import skip_in_parallel

def test_first_shape_derivative():
    mesh = UnitSquareMesh(6, 6)
    n = FacetNormal(mesh)
    X = SpatialCoordinate(mesh)
    x, y = X
    V = FunctionSpace(mesh, "Lagrange", 1)
    u = project(x*x+y*x+y*y+sin(x)+cos(x), V)
    dX = TestFunction(VectorFunctionSpace(mesh, "Lagrange", 1))

    def test_first(J, dJ):
        computed = assemble(derivative(J, X)).get_local()
        actual = assemble(dJ).get_local()
        assert np.allclose(computed, actual, rtol=1e-14)

    Ja = u * u * dx
    dJa = u * u * div(dX) * dx
    test_first(Ja, dJa)

    Jb = inner(grad(u), grad(u)) * dx
    dJb = -2*inner(dot(grad(dX), grad(u)), grad(u)) * dx + inner(grad(u), grad(u)) * div(dX) * dx
    test_first(Jb, dJb)

    f = x * y * sin(x) * cos(y)
    Jc = f * dx
    dJc = div(f * dX) * dx
    test_first(Jc, dJc)

    Jd = f * ds
    dJd = inner(grad(f), dX) * ds \
        + f * (div(dX) - inner(dot(grad(dX), n), n)) * ds
    test_first(Jd, dJd)

    J = Ja + Jb + Jc + Jd
    dJ = dJa + dJb + dJc + dJd
    test_first(J, dJ)


def test_mixed_derivatives():
    mesh = UnitSquareMesh(6, 6)
    X = SpatialCoordinate(mesh)
    x, y = X
    V = FunctionSpace(mesh, "CG", 1)
    u = project(x * x + y * x + y * y + sin(x) + cos(x), V)
    v = TrialFunction(V)
    dX = TestFunction(VectorFunctionSpace(mesh, "Lagrange", 1))
    dX_ = TrialFunction(VectorFunctionSpace(mesh, "Lagrange", 1))


    def test_mixed(J, dJ_manual):
        computed1 = assemble(derivative(derivative(J, X, dX), u)).norm("frobenius")
        computed2 = assemble(derivative(derivative(J, u), X, dX_)).norm("frobenius")
        computed3 = assemble(derivative(derivative(J, X), u)).norm("frobenius")
        computed4 = assemble(derivative(derivative(J, u), X)).norm("frobenius")
        actuala = assemble(dJ_manual).norm("frobenius")
        assert np.isclose(computed1, actuala, rtol=1e-14)
        assert np.isclose(computed2, actuala, rtol=1e-14)
        assert np.isclose(computed3, actuala, rtol=1e-14)
        assert np.isclose(computed4, actuala, rtol=1e-14)

    Ja = u * u * dx
    dJa = 2 * u * v * div(dX) * dx
    test_mixed(Ja, dJa)

    Jb = inner(grad(u), grad(u)) * dx
    dJb = 2*inner(grad(u), grad(v)) * div(dX) * dx \
        - 2*inner(dot(nabla_grad(dX), grad(u)), grad(v)) * dx \
        - 2*inner(grad(u), dot(nabla_grad(dX), grad(v))) * dx
    test_mixed(Jb, dJb)

    J = Ja + Jb
    dJ = dJa + dJb
    test_mixed(J, dJ)



def test_second_shape_derivative():
    mesh = UnitSquareMesh(6, 6)
    V = FunctionSpace(mesh, "CG", 1)
    X = SpatialCoordinate(mesh)
    x, y = X
    u = project(x * x + y * x + y * y + sin(x) + cos(x), V)
    Z = VectorFunctionSpace(mesh, "Lagrange", 1)
    dX1 = TestFunction(Z)
    dX2 = TrialFunction(Z)

    def test_second(J, ddJ):
        computed = assemble(derivative(derivative(J, X, dX1), X, dX2)).norm("frobenius")
        actual = assemble(ddJ).norm("frobenius")
        assert np.isclose(computed, actual, rtol=1e-14)

    Ja = u * u * dx
    ddJa = u * u * div(dX1) * div(dX2) * dx - u * u * tr(grad(dX1)*grad(dX2)) * dx
    test_second(Ja, ddJa)

    Jb = inner(grad(u), grad(u)) * dx
    ddJb = 2*inner(dot(dot(nabla_grad(dX2), nabla_grad(dX1)), grad(u)), grad(u)) * dx \
        + 2*inner(dot(nabla_grad(dX1), dot(nabla_grad(dX2), grad(u))), grad(u)) * dx \
        + 2*inner(dot(nabla_grad(dX1), grad(u)), dot(nabla_grad(dX2), grad(u))) * dx \
        - 2*inner(dot(nabla_grad(dX2), grad(u)), grad(u)) * div(dX1) * dx \
        - inner(grad(u), grad(u)) * tr(nabla_grad(dX1)*nabla_grad(dX2)) * dx \
        - 2*inner(dot(nabla_grad(dX1), grad(u)), grad(u)) * div(dX2) * dx \
        + inner(grad(u), grad(u)) * div(dX1) * div(dX2) * dx
    test_second(Jb, ddJb)

    test_second(Ja+Jb, ddJa + ddJb)


# In parallel a RuntimeError is thrown instead of the specific
# UFLException, so we skip that case.
@skip_in_parallel
def test_integral_scaling_edge_case():
    mesh = UnitSquareMesh(6, 6)
    X = SpatialCoordinate(mesh)
    V = FunctionSpace(mesh, "CG", 1)
    u = Function(V)

    J = u * u * dx

    with pytest.raises(UFLException):
        assemble(Constant(2.0) * derivative(J, X))
    with pytest.raises(UFLException):
        assemble(derivative(Constant(2.0) * derivative(J, X), X))
    with pytest.raises(UFLException):
        assemble(Constant(2.0) * derivative(derivative(J, X), X))


def test_different_quadratures():
    """
    Checking that expressions with different quadrature rules
    can be assembled as a sum, using the higher order quadrature for all
    terms
    """
    mesh = UnitIntervalMesh(10)
    x = SpatialCoordinate(mesh)
    f = x[0]**2
    V = FunctionSpace(mesh, "CG", 1)
    u, v = TrialFunction(V), TestFunction(V)
    a = inner(grad(u), grad(v)) * dx + inner(u, v) * dx
    l = f * v * dx
    bc = DirichletBC(V, Constant(1), "on_boundary")
    uh = Function(V)
    solve(a==l, uh, bcs=bc)
    F = a - l
    J = uh**2 * dx

    # Adjoint equation disregarding Dirichlet BC adjoint
    dJdu = derivative(J, uh)
    dFdu = derivative(action(F, uh), uh)
    bc_hom = DirichletBC(V, Constant(0), "on_boundary")
    lmbd = Function(V)
    solve(dFdu==-dJdu, lmbd, bc_hom)
    F = replace(F, {u: uh})
    dJdu = replace(dJdu, {u: uh})

    # Parts of second order adjoint equation:
    S = VectorFunctionSpace(mesh, "CG", 1)
    s = Function(S)
    F = replace(F, {v: lmbd})
    ds = TestFunction(S)
    dFdm = derivative(F, x, ds)
    d2Fdm2 = derivative(dFdm, x, s)
    soas_separated = assemble(d2Fdm2) + assemble(dFdm)

    soa = d2Fdm2 + dFdm
    soas = assemble(soa)
    assert np.isclose(soas_separated.norm("l2"), soas.norm("l2"))

    # 2D test for triangles with quadrature degree 3 and 4
    mesh = UnitSquareMesh(5, 5)
    x = SpatialCoordinate(mesh)
    V = FunctionSpace(mesh, "CG", 1)
    u0 = cos(pi*x[0])*sin(pi*x[1])

    u, v = TrialFunction(V), TestFunction(V)
    F = inner(u - u0, v)*dx
    uh = Function(V)
    solve(lhs(F) == rhs(F), uh)
    J = uh**2*dx

    # adjoint eq disregarding Dirichlet BC
    dJdu = derivative(J, uh)
    dFdu = derivative(action(F, uh), uh)
    bc_hom = DirichletBC(V, Constant(0), "on_boundary")
    lmbd = Function(V)
    solve(dFdu==-dJdu, lmbd, bc_hom)
    F = replace(F, {u:uh})
    dJdu = replace(dJdu, {u: uh})
    # Parts of second order adjoint eq:
    S = VectorFunctionSpace(mesh, "CG", 1)
    s = Function(S)
    F = replace(F, {v: lmbd})
    ds = TestFunction(S)
    dFdm = derivative(F, x, ds)
    d2Fdm2 = derivative(dFdm, x, s)
    d2Fdudm = derivative(dFdm, uh, lmbd)
    soas_separated = assemble(d2Fdm2) + assemble(dFdm) + assemble(d2Fdudm)
    soa = d2Fdm2 + d2Fdudm + dFdm
    soas = assemble(soa)
    assert np.isclose(soas_separated.norm("l2"), soas.norm("l2"))


def test_unique_tables():
    """
    Checking that the unique table names in FFC are cached correctly
    """
    mesh = UnitIntervalMesh(10)
    x = SpatialCoordinate(mesh)
    f = x[0]**2
    V = FunctionSpace(mesh, "CG", 1)
    f = project(f, V)
    u, v = TrialFunction(V), TestFunction(V)
    a = inner(grad(u), grad(v)) * dx + inner(u, v) * dx
    l = f * v * dx
    bc = DirichletBC(V, Constant(1), "on_boundary")
    uh = Function(V)
    solve(a == l, uh, bcs=bc)
    F = a - l
    J = uh**2 * dx

    # Adjoint eq disregarding Dirichlet BC
    dJdu = derivative(J, uh)
    dFdu = derivative(action(F, uh), uh)
    bc_hom = DirichletBC(V, Constant(0), "on_boundary")
    lmbd = Function(V)
    solve(dFdu == -dJdu, lmbd, bc_hom)
    F = replace(F, {u: uh})
    dJdu = replace(dJdu, {u: uh})

    # Parts of second order adjoint eq:
    S = VectorFunctionSpace(mesh, "CG", 1)
    s = Function(S)
    F = replace(F, {v: lmbd})
    ds = TestFunction(S)
    dFdm = derivative(F, x, ds)
    d2Fdm2 = derivative(dFdm, x, s)
    soas_separated = assemble(d2Fdm2) + assemble(dFdm)

    soa = d2Fdm2 + dFdm
    soas = assemble(soa)
    assert np.isclose(soas_separated.norm("l2"), soas.norm("l2"))


if __name__ == "__main__":
    import os
    pytest.main(os.path.abspath(__file__))
